@@ -16,82 +16,79 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-  *
+ *
  */
+#include "internal.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-
-#include <gtk/gtk.h>
-#include "gaim.h"
+#include "conversation.h"
+#include "debug.h"
+#include "plugin.h"
+#include "prefs.h"
 #include "prpl.h"
+#include "util.h"
+
+/* XXX CORE/UI: Until we can get rid of the message queue stuff... */
+#include "gaim.h"
+#include "gtkinternal.h"
+#include "gtkblist.h"
 #include "gtkimhtml.h"
-#include "pixmaps/join.xpm"
+#include "gtkprefs.h"
+#include "gtkutils.h"
+#include "ui.h"
 
 GtkWidget *imaway = NULL;
-
 GtkWidget *awaymenu = NULL;
 GtkWidget *awayqueue = NULL;
 GtkListStore *awayqueuestore = NULL;
 GtkWidget *awayqueuesw;
 
+GSList *message_queue = NULL;
+GSList *unread_message_queue = NULL;
+
+GSList *away_messages = NULL;
 struct away_message *awaymessage = NULL;
-struct away_message *default_away;
-int auto_away;
-
-static void destroy_im_away()
-{
-	if (imaway)
-		gtk_widget_destroy(imaway);
-
-	awayqueue = NULL;
-	g_object_unref(G_OBJECT(awayqueuestore));
-	awayqueuestore = NULL;
-	awayqueuesw = NULL;
-	imaway = NULL;
-}
 
 static void dequeue_message(GtkTreeIter *iter)
 {
 	gchar *name;
 	GSList *templist;
-	struct gaim_conversation *cnv;
+	GaimConversation *cnv;
 
 	gtk_tree_model_get(GTK_TREE_MODEL(awayqueuestore), iter, 0, &name, -1);
-	
-	debug_printf("Unqueueing messages from %s.\n", name);
-	
+
+	gaim_debug(GAIM_DEBUG_INFO, "away", "Unqueueing messages from %s.\n",
+			   name);
+
 	templist = message_queue;
-	
+
 	while (templist) {
 		struct queued_message *qm = templist->data;
 		if (templist->data) {
-			if (!g_strcasecmp(qm->name, name)) {
-				cnv = gaim_find_conversation(name);
+			if (!gaim_utf8_strcasecmp(qm->name, name)) {
+				GaimAccount *account = NULL;
+
+				if (g_list_index(gaim_accounts_get_all(), qm->account) >= 0)
+					account = qm->account;
+
+				cnv = gaim_find_conversation_with_account(name, account);
 
 				if (!cnv)
-					cnv = gaim_conversation_new(GAIM_CONV_IM, qm->name);
-
-				if (g_slist_index(connections, qm->gc) >= 0)
-					gaim_conversation_set_user(cnv, qm->gc->user);
+					cnv = gaim_conversation_new(GAIM_CONV_IM, account, qm->name);
+				else
+					gaim_conversation_set_account(cnv, account);
 
 				gaim_im_write(GAIM_IM(cnv), NULL, qm->message, qm->len,
 						qm->flags, qm->tm);
 				g_free(qm->message);
 				g_free(qm);
 				templist = message_queue = g_slist_remove(message_queue, qm);
-				
+
 			} else {
 				templist = templist->next;
 			}
 		}
 	}
-	
+
 	g_free(name);
 	/* In GTK 2.2, _store_remove actually returns whether iter is valid or not
 	 * after the remove, but in GTK 2.0 it is a void function. */
@@ -102,18 +99,23 @@ void purge_away_queue(GSList **queue)
 {
 	GSList *q = *queue;
 	struct queued_message *qm;
-	struct gaim_conversation *cnv;
+	GaimConversation *cnv;
+	GaimAccount *account;
 
 	while (q) {
 		qm = q->data;
 
-		cnv = gaim_find_conversation(qm->name);
+		account = NULL;
+
+		if (g_list_index(gaim_accounts_get_all(), qm->account) >= 0)
+			account = qm->account;
+
+		cnv = gaim_find_conversation_with_account(qm->name, account);
 
 		if (!cnv)
-			cnv = gaim_conversation_new(GAIM_CONV_IM, qm->name);
-
-		if (g_slist_index(connections, qm->gc) >= 0)
-			gaim_conversation_set_user(cnv, qm->gc->user);
+			cnv = gaim_conversation_new(GAIM_CONV_IM, account, qm->name);
+		else
+			gaim_conversation_set_account(cnv, account);
 
 		gaim_im_write(GAIM_IM(cnv), NULL, qm->message, -1, qm->flags, qm->tm);
 
@@ -131,17 +133,17 @@ void purge_away_queue(GSList **queue)
 gint dequeue_cb(GtkWidget *treeview, GdkEventButton *event, gpointer data) {
 	GtkTreeIter iter;
 	GtkTreeSelection *select;
-	
+
 	if(!(event->type == GDK_2BUTTON_PRESS && event->button == 1))
 		return FALSE; /* Double clicking on the list will unqueue that user's messages. */
-	
+
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
 	if(gtk_tree_selection_get_selected(select, NULL, &iter))
 			dequeue_message(&iter);
 
 	return FALSE;
 }
-	
+
 
 
 void toggle_away_queue()
@@ -149,7 +151,7 @@ void toggle_away_queue()
 	if (!awayqueue || !awayqueuesw)
 		return;
 
-	if (away_options & OPT_AWAY_QUEUE) {
+	if (gaim_prefs_get_bool("/gaim/gtk/away/queue_messages")) {
 		gtk_widget_show(awayqueue);
 		gtk_widget_show(awayqueuesw);
 	} else {
@@ -172,12 +174,6 @@ void do_im_back(GtkWidget *w, GtkWidget *x)
 			return;
 	}
 
-	while (away_time_queue) {
-		struct queued_away_response *qar = away_time_queue->data;
-		away_time_queue = g_slist_remove(away_time_queue, qar);
-		g_free(qar);
-	}
-
 	awaymessage = NULL;
 	awayqueue = NULL;
 	awayqueuesw = NULL;
@@ -195,132 +191,118 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 	GtkWidget *vbox;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
-	char *buf2;
 	char *buf;
 
-	if (!blist)
-		return;
-
 	if (!a)
 		return;
 
-	if (!imaway) {
-		GAIM_DIALOG(imaway);
-		gtk_window_set_role(GTK_WINDOW(imaway), "imaway");
-		if (strlen(a->name))
-			 gtk_window_set_title(GTK_WINDOW(imaway), a->name);
-		else
-			gtk_window_set_title(GTK_WINDOW(imaway), _("Gaim - Away!"));
-		g_signal_connect(G_OBJECT(imaway), "destroy",
-						 G_CALLBACK(do_im_back), imaway);
-		gtk_widget_realize(imaway);
+	if(imaway)
+		gtk_widget_destroy(imaway);
 
-		vbox = gtk_vbox_new(FALSE, 5);
-		gtk_container_add(GTK_CONTAINER(imaway), vbox);
-		gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-		gtk_widget_show(vbox);
+	GAIM_DIALOG(imaway);
+	gtk_window_set_role(GTK_WINDOW(imaway), "imaway");
+	if (strlen(a->name))
+		gtk_window_set_title(GTK_WINDOW(imaway), a->name);
+	else
+		gtk_window_set_title(GTK_WINDOW(imaway), _("Gaim - Away!"));
+	g_signal_connect(G_OBJECT(imaway), "destroy",
+			G_CALLBACK(do_im_back), imaway);
+	gtk_widget_realize(imaway);
 
-		sw = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER,
-					       GTK_POLICY_ALWAYS);
-		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
-		gtk_widget_set_usize(sw, 245, 120);
-		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
-		gtk_widget_show(sw);
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_container_add(GTK_CONTAINER(imaway), vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+	gtk_widget_show(vbox);
 
-		awaytext = gtk_imhtml_new(NULL, NULL);
-		gtk_container_add(GTK_CONTAINER(sw), awaytext);
-		gaim_setup_imhtml(awaytext);
-		gtk_widget_show(awaytext);
-		buf = stylize(a->message, BUF_LONG);
-		gtk_imhtml_append_text(GTK_IMHTML(awaytext), buf, -1, GTK_IMHTML_NO_TITLE |
-				       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
-		g_free(buf);
-		gtk_imhtml_append_text(GTK_IMHTML(awaytext), "<BR>", -1, GTK_IMHTML_NO_TITLE |
-				       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
+	sw = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER,
+			GTK_POLICY_ALWAYS);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
+	gtk_widget_set_size_request(sw, 245, 120);
+	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+	gtk_widget_show(sw);
 
-		awayqueuesw = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(awayqueuesw), GTK_POLICY_NEVER,
-					       GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(awayqueuesw),
-							GTK_SHADOW_IN);
-		gtk_box_pack_start(GTK_BOX(vbox), awayqueuesw, TRUE, TRUE, 0);
+	awaytext = gtk_imhtml_new(NULL, NULL);
+	gtk_container_add(GTK_CONTAINER(sw), awaytext);
+	gaim_setup_imhtml(awaytext);
+	gtk_widget_show(awaytext);
+	buf = stylize(a->message, BUF_LONG);
+	gtk_imhtml_append_text(GTK_IMHTML(awaytext), buf, -1, GTK_IMHTML_NO_TITLE |
+			GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
+	g_free(buf);
+	gtk_imhtml_append_text(GTK_IMHTML(awaytext), "<BR>", -1,
+			GTK_IMHTML_NO_TITLE | GTK_IMHTML_NO_COMMENTS |
+			GTK_IMHTML_NO_SCROLL);
 
-		awayqueuestore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
-		awayqueue = gtk_tree_view_new_with_model(GTK_TREE_MODEL(awayqueuestore));
-		renderer = gtk_cell_renderer_text_new();
-		
-		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(awayqueue), FALSE);
-		column = gtk_tree_view_column_new_with_attributes (NULL, renderer,
-										"text", 0,
-										NULL);
-		gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
-		column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
-										"text", 1, 
-										NULL);
-		gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
-			
-		gtk_container_add(GTK_CONTAINER(awayqueuesw), awayqueue);
-		
-		g_signal_connect(G_OBJECT(awayqueue), "button_press_event", G_CALLBACK(dequeue_cb), NULL);
+	awayqueuesw = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(awayqueuesw),
+			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(awayqueuesw),
+			GTK_SHADOW_IN);
+	gtk_box_pack_start(GTK_BOX(vbox), awayqueuesw, TRUE, TRUE, 0);
+
+	awayqueuestore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	awayqueue = gtk_tree_view_new_with_model(GTK_TREE_MODEL(awayqueuestore));
+	renderer = gtk_cell_renderer_text_new();
+
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(awayqueue), FALSE);
+	column = gtk_tree_view_column_new_with_attributes (NULL, renderer,
+			"text", 0,
+			NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
+	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
+			"text", 1,
+			NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
+
+	gtk_container_add(GTK_CONTAINER(awayqueuesw), awayqueue);
+
+	g_signal_connect(G_OBJECT(awayqueue), "button_press_event", G_CALLBACK(dequeue_cb), NULL);
 
 
-		if (away_options & OPT_AWAY_QUEUE) {
-			gtk_widget_show(awayqueuesw);
-			gtk_widget_show(awayqueue);
-		}
-
-		back = picture_button(imaway, _("I'm Back!"), join_xpm);
-		gtk_box_pack_start(GTK_BOX(vbox), back, FALSE, FALSE, 0);
-		g_signal_connect(GTK_OBJECT(back), "clicked", G_CALLBACK(do_im_back), imaway);
-		gtk_window_set_focus(GTK_WINDOW(imaway), back);
-
-		awaymessage = a;
-
-#ifdef _WIN32
-		/* Register window widget with wgaim systray module */
-		wgaim_created_backwin(imaway);
-#endif
-
-	} else {
-		destroy_im_away();
-		do_away_message(w, a);
-		return;
+	if (gaim_prefs_get_bool("/gaim/gtk/away/queue_messages")) {
+		gtk_widget_show(awayqueuesw);
+		gtk_widget_show(awayqueue);
 	}
 
-	/* New away message... Clear out the old sent_aways */
-	while (away_time_queue) {
-		struct queued_away_response *qar = away_time_queue->data;
-		away_time_queue = g_slist_remove(away_time_queue, qar);
-		g_free(qar);
-	}
+	back = gaim_pixbuf_button_from_stock(_("I'm Back!"), GTK_STOCK_JUMP_TO, GAIM_BUTTON_HORIZONTAL);
+	gtk_box_pack_start(GTK_BOX(vbox), back, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(back), "clicked", G_CALLBACK(do_im_back), imaway);
+	gtk_window_set_focus(GTK_WINDOW(imaway), back);
+	gtk_widget_show(back);
+
+	awaymessage = a;
 
 	gtk_widget_show(imaway);
-	buf2 = g_malloc(strlen(awaymessage->message) * 4 + 1);
-	strncpy_withhtml(buf2, awaymessage->message, strlen(awaymessage->message) * 4 + 1);
-	serv_set_away_all(buf2);
-	g_free(buf2);
+	serv_set_away_all(awaymessage->message);
 }
 
 void rem_away_mess(GtkWidget *w, struct away_message *a)
 {
-	int default_index;
-	default_index = g_slist_index(away_messages, default_away);
-	if (default_index == -1) {
-		if (away_messages != NULL)
-			default_away = away_messages->data;
-		else
-			default_away = NULL;
+	struct away_message *default_away = NULL;
+	const char *default_away_name;
+	GSList *l;
+
+	default_away_name = gaim_prefs_get_string("/core/away/default_message");
+
+	for(l = away_messages; l; l = l->next) {
+		if(!strcmp(default_away_name, ((struct away_message *)l->data)->name)) {
+			default_away = l->data;
+			break;
+		}
 	}
+
+	if(!default_away && away_messages)
+		default_away = away_messages->data;
+
 	away_messages = g_slist_remove(away_messages, a);
 	g_free(a);
 	do_away_menu();
-	save_prefs();
 }
 
-static void set_gc_away(GtkObject *obj, struct gaim_connection *gc)
+static void set_gc_away(GObject *obj, GaimConnection *gc)
 {
-	struct away_message *awy = gtk_object_get_user_data(obj);
+	struct away_message *awy = g_object_get_data(obj, "away_message");
 
 	if (awy)
 		serv_set_away(gc, GAIM_AWAY_CUSTOM, awy->message);
@@ -328,57 +310,50 @@ static void set_gc_away(GtkObject *obj, struct gaim_connection *gc)
 		serv_set_away(gc, GAIM_AWAY_CUSTOM, NULL);
 }
 
-static void set_gc_state(GtkObject *obj, struct gaim_connection *gc)
+static void set_gc_state(GObject *obj, GaimConnection *gc)
 {
-	char *awy = gtk_object_get_user_data(obj);
+	char *awy = g_object_get_data(obj, "away_state");
 
 	serv_set_away(gc, awy, NULL);
 }
 
+/* XXX This needs to be fixed, NOW! */
+extern GtkListStore *prefs_away_store;
+extern GtkWidget *prefs_away_menu;
+
 void do_away_menu()
 {
 	GtkWidget *menuitem;
 	GtkWidget *remmenu;
 	GtkWidget *submenu, *submenu2;
 	GtkWidget *remitem;
-	GtkWidget *label;
-	GtkWidget *sep;
+	GtkWidget *image;
+	GdkPixbuf *pixbuf, *scale;
 	GList *l;
-	GtkWidget *list_item;
 	GSList *awy = away_messages;
 	struct away_message *a;
-	GSList *con = connections;
-	struct gaim_connection *gc = NULL;
+	GList *con;
+	GaimConnection *gc = NULL;
+	GaimPluginProtocolInfo *prpl_info = NULL;
+
 	int count = 0;
 
-	if (prefs_away_list != NULL) {
-		GtkWidget *hbox;
-		gtk_list_clear_items(GTK_LIST(prefs_away_list), 0, -1);
+	if (prefs_away_store != NULL) {
+		gtk_list_store_clear(prefs_away_store);
 		while (awy) {
+			GtkTreeIter iter;
 			a = (struct away_message *)awy->data;
-			list_item = gtk_list_item_new();
-			gtk_container_add(GTK_CONTAINER(prefs_away_list), list_item);
-			g_signal_connect(GTK_OBJECT(list_item), "select",
-					   G_CALLBACK(away_list_clicked), a);
-			gtk_object_set_user_data(GTK_OBJECT(list_item), a);
-			gtk_widget_show(list_item);
-
-			hbox = gtk_hbox_new(FALSE, 5);
-			gtk_container_add(GTK_CONTAINER(list_item), hbox);
-			gtk_widget_show(hbox);
-
-			label = gtk_label_new(a->name);
-			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-			gtk_widget_show(label);
-
+			gtk_list_store_append(prefs_away_store, &iter);
+			gtk_list_store_set(prefs_away_store, &iter,
+					0, a->name,
+					1, a,
+					-1);
 			awy = g_slist_next(awy);
 		}
-		if (away_messages != NULL)
-			gtk_list_select_item(GTK_LIST(prefs_away_list), 0);
 	}
 
 	if (awaymenu) {
-		l = gtk_container_children(GTK_CONTAINER(awaymenu));
+		l = gtk_container_get_children(GTK_CONTAINER(awaymenu));
 
 		while (l) {
 			gtk_container_remove(GTK_CONTAINER(awaymenu), GTK_WIDGET(l->data));
@@ -389,9 +364,9 @@ void do_away_menu()
 		remmenu = gtk_menu_new();
 
 		menuitem = gtk_menu_item_new_with_label(_("New Away Message"));
-		gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+		gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 		gtk_widget_show(menuitem);
-		g_signal_connect(GTK_OBJECT(menuitem), "activate", G_CALLBACK(create_away_mess),
+		g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(create_away_mess),
 				   NULL);
 
 		awy = away_messages;
@@ -399,9 +374,9 @@ void do_away_menu()
 			a = (struct away_message *)awy->data;
 
 			remitem = gtk_menu_item_new_with_label(a->name);
-			gtk_menu_append(GTK_MENU(remmenu), remitem);
+			gtk_menu_shell_append(GTK_MENU_SHELL(remmenu), remitem);
 			gtk_widget_show(remitem);
-			g_signal_connect(GTK_OBJECT(remitem), "activate",
+			g_signal_connect(G_OBJECT(remitem), "activate",
 					   G_CALLBACK(rem_away_mess), a);
 
 			awy = g_slist_next(awy);
@@ -409,38 +384,36 @@ void do_away_menu()
 		}
 
 		menuitem = gtk_menu_item_new_with_label(_("Remove Away Message"));
-		gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+		gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 		gtk_widget_show(menuitem);
 		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), remmenu);
 		gtk_widget_show(remmenu);
 
-		sep = gtk_hseparator_new();
-		menuitem = gtk_menu_item_new();
-		gtk_menu_append(GTK_MENU(awaymenu), menuitem);
-		gtk_container_add(GTK_CONTAINER(menuitem), sep);
-		gtk_widget_set_sensitive(menuitem, FALSE);
-		gtk_widget_show(menuitem);
-		gtk_widget_show(sep);
+		gaim_separator(awaymenu);
 
-		while (con) {
+		for (con = gaim_connections_get_all(); con != NULL; con = con->next) {
 			gc = con->data;
-			if (gc->prpl->away_states &&gc->prpl->set_away)
+
+			prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+			if (prpl_info->away_states != NULL && prpl_info->set_away != NULL)
 				count++;
-			con = g_slist_next(con);
 		}
-		con = connections;
 
 		if (count == 0) {
 		} else if (count == 1) {
 			GList *msgs, *tmp;
-			while (con) {
+
+			for (con = gaim_connections_get_all(); con != NULL; con = con->next) {
 				gc = con->data;
-				if (gc->prpl->away_states &&gc->prpl->set_away)
+
+				prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+				if (prpl_info->away_states && prpl_info->set_away)
 					break;
-				con = g_slist_next(con);
 			}
 
-			tmp = msgs = gc->prpl->away_states(gc);
+			tmp = msgs = prpl_info->away_states(gc);
 
 			if ((g_list_length(msgs) == 1) && !strcmp(msgs->data, GAIM_AWAY_CUSTOM)) {
 				awy = away_messages;
@@ -449,10 +422,10 @@ void do_away_menu()
 					a = (struct away_message *)awy->data;
 
 					menuitem = gtk_menu_item_new_with_label(a->name);
-					gtk_object_set_user_data(GTK_OBJECT(menuitem), a);
-					gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+					g_object_set_data(G_OBJECT(menuitem), "away_message", a);
+					gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 					gtk_widget_show(menuitem);
-					g_signal_connect(GTK_OBJECT(menuitem), "activate",
+					g_signal_connect(G_OBJECT(menuitem), "activate",
 							   G_CALLBACK(do_away_message), a);
 
 					awy = g_slist_next(awy);
@@ -462,12 +435,12 @@ void do_away_menu()
 					awy = away_messages;
 
 					menuitem = gtk_menu_item_new_with_label(msgs->data);
-					gtk_object_set_user_data(GTK_OBJECT(menuitem), msgs->data);
-					gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+					g_object_set_data(G_OBJECT(menuitem), "away_state", msgs->data);
+					gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 					gtk_widget_show(menuitem);
 
 					if (strcmp(msgs->data, GAIM_AWAY_CUSTOM)) {
-						g_signal_connect(GTK_OBJECT(menuitem), "activate",
+						g_signal_connect(G_OBJECT(menuitem), "activate",
 								   G_CALLBACK(set_gc_state), gc);
 					} else {
 						submenu = gtk_menu_new();
@@ -479,11 +452,12 @@ void do_away_menu()
 							a = (struct away_message *)awy->data;
 
 							menuitem = gtk_menu_item_new_with_label(a->name);
-							gtk_object_set_user_data(GTK_OBJECT(menuitem),
-										 a);
-							gtk_menu_append(GTK_MENU(submenu), menuitem);
+							g_object_set_data(G_OBJECT(menuitem), "away_message",
+									a);
+							gtk_menu_shell_append(GTK_MENU_SHELL(submenu),
+									menuitem);
 							gtk_widget_show(menuitem);
-							g_signal_connect(GTK_OBJECT(menuitem),
+							g_signal_connect(G_OBJECT(menuitem),
 									   "activate",
 									   G_CALLBACK
 									   (do_away_message), a);
@@ -495,44 +469,54 @@ void do_away_menu()
 				}
 
 			g_list_free(tmp);
-		} else {
-			while (con) {
+		}
+		else {
+			for (con = gaim_connections_get_all(); con != NULL; con = con->next) {
+				GaimAccount *account;
 				char buf[256];
 				GList *msgs, *tmp;
 				gc = con->data;
 
-				if (!gc->prpl->away_states ||!gc->prpl->set_away) {
-					con = con->next;
+				prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+				if (!prpl_info->away_states || !prpl_info->set_away)
 					continue;
-				}
+
+				account = gaim_connection_get_account(gc);
 
 				g_snprintf(buf, sizeof(buf), "%s (%s)",
-					   gc->username, gc->prpl->name);
-				menuitem = gtk_menu_item_new_with_label(buf);
-				gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+					   gaim_account_get_username(account), gc->prpl->info->name);
+				menuitem = gtk_image_menu_item_new_with_label(buf);
+
+				pixbuf = create_prpl_icon(gc->account);
+				if (pixbuf) {
+					scale = gdk_pixbuf_scale_simple(pixbuf, 16, 16, GDK_INTERP_BILINEAR);
+					image = gtk_image_new_from_pixbuf(scale);
+					g_object_unref(G_OBJECT(pixbuf));
+					g_object_unref(G_OBJECT(scale));
+					gtk_widget_show(image);
+					gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),
+							image);
+				}
+
+				gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 				gtk_widget_show(menuitem);
 
 				submenu = gtk_menu_new();
 				gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
 				gtk_widget_show(submenu);
 
-				tmp = msgs = gc->prpl->away_states(gc);
+				tmp = msgs = prpl_info->away_states(gc);
 
 				if ((g_list_length(msgs) == 1) &&
 				    (!strcmp(msgs->data, GAIM_AWAY_CUSTOM))) {
 					menuitem = gtk_menu_item_new_with_label(_("Back"));
-					gtk_menu_append(GTK_MENU(submenu), menuitem);
+					gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
 					gtk_widget_show(menuitem);
-					g_signal_connect(GTK_OBJECT(menuitem), "activate",
+					g_signal_connect(G_OBJECT(menuitem), "activate",
 							   G_CALLBACK(set_gc_away), gc);
 
-					sep = gtk_hseparator_new();
-					menuitem = gtk_menu_item_new();
-					gtk_menu_append(GTK_MENU(submenu), menuitem);
-					gtk_container_add(GTK_CONTAINER(menuitem), sep);
-					gtk_widget_set_sensitive(menuitem, FALSE);
-					gtk_widget_show(menuitem);
-					gtk_widget_show(sep);
+					gaim_separator(submenu);
 
 					awy = away_messages;
 
@@ -540,10 +524,10 @@ void do_away_menu()
 						a = (struct away_message *)awy->data;
 
 						menuitem = gtk_menu_item_new_with_label(a->name);
-						gtk_object_set_user_data(GTK_OBJECT(menuitem), a);
-						gtk_menu_append(GTK_MENU(submenu), menuitem);
+						g_object_set_data(G_OBJECT(menuitem), "away_message", a);
+						gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
 						gtk_widget_show(menuitem);
-						g_signal_connect(GTK_OBJECT(menuitem), "activate",
+						g_signal_connect(G_OBJECT(menuitem), "activate",
 								   G_CALLBACK(set_gc_away), gc);
 
 						awy = g_slist_next(awy);
@@ -553,13 +537,13 @@ void do_away_menu()
 						awy = away_messages;
 
 						menuitem = gtk_menu_item_new_with_label(msgs->data);
-						gtk_object_set_user_data(GTK_OBJECT(menuitem),
+						g_object_set_data(G_OBJECT(menuitem), "away_state",
 									 msgs->data);
-						gtk_menu_append(GTK_MENU(submenu), menuitem);
+						gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
 						gtk_widget_show(menuitem);
 
 						if (strcmp(msgs->data, GAIM_AWAY_CUSTOM)) {
-							g_signal_connect(GTK_OBJECT(menuitem),
+							g_signal_connect(G_OBJECT(menuitem),
 									   "activate",
 									   G_CALLBACK(set_gc_state),
 									   gc);
@@ -575,12 +559,12 @@ void do_away_menu()
 								menuitem =
 								    gtk_menu_item_new_with_label(a->
 												 name);
-								gtk_object_set_user_data(GTK_OBJECT
-											 (menuitem), a);
-								gtk_menu_append(GTK_MENU(submenu2),
+								g_object_set_data(G_OBJECT(menuitem),
+										"away_message", a);
+								gtk_menu_shell_append(GTK_MENU_SHELL(submenu2),
 										menuitem);
 								gtk_widget_show(menuitem);
-								g_signal_connect(GTK_OBJECT(menuitem),
+								g_signal_connect(G_OBJECT(menuitem),
 										   "activate",
 										   G_CALLBACK
 										   (set_gc_away), gc);
@@ -592,12 +576,10 @@ void do_away_menu()
 					}
 
 				g_list_free(tmp);
-
-				con = g_slist_next(con);
 			}
 
 			menuitem = gtk_menu_item_new_with_label(_("Set All Away"));
-			gtk_menu_append(GTK_MENU(awaymenu), menuitem);
+			gtk_menu_shell_append(GTK_MENU_SHELL(awaymenu), menuitem);
 			gtk_widget_show(menuitem);
 
 			submenu = gtk_menu_new();
@@ -610,18 +592,19 @@ void do_away_menu()
 				a = (struct away_message *)awy->data;
 
 				menuitem = gtk_menu_item_new_with_label(a->name);
-				gtk_object_set_user_data(GTK_OBJECT(menuitem), a);
-				gtk_menu_append(GTK_MENU(submenu), menuitem);
+				g_object_set_data(G_OBJECT(menuitem), "away_message", a);
+				gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
 				gtk_widget_show(menuitem);
-				g_signal_connect(GTK_OBJECT(menuitem), "activate",
+				g_signal_connect(G_OBJECT(menuitem), "activate",
 						   G_CALLBACK(do_away_message), a);
 
 				awy = g_slist_next(awy);
 			}
 		}
 	}
+
 	if (prefs_away_menu) {
-		l = gtk_container_children(GTK_CONTAINER(prefs_away_menu));
+		l = gtk_container_get_children(GTK_CONTAINER(prefs_away_menu));
 		while (l) {
 			gtk_widget_destroy(GTK_WIDGET(l->data));
 			l = l->next;
