@@ -19,28 +19,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-#include <string.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#ifndef _WIN32
-#include <unistd.h>
-#else
-#include <direct.h>
-#endif
-#include <ctype.h>
-#include "gaim.h"
-#include "prpl.h"
+#include "internal.h"
 #include "blist.h"
+#include "conversation.h"
+#include "debug.h"
+#include "multi.h"
 #include "notify.h"
-
-#ifdef _WIN32
-#include "win32dep.h"
-#endif
+#include "prefs.h"
+#include "privacy.h"
+#include "prpl.h"
+#include "server.h"
+#include "signals.h"
+#include "util.h"
 
 #define PATHSIZE 1024
 
@@ -68,7 +58,8 @@ static GaimBlistNode *gaim_blist_get_last_child(GaimBlistNode *node)
 
 struct _gaim_hbuddy {
 	char *name;
-	struct gaim_account *account;
+	GaimAccount *account;
+	GaimBlistNode *group;
 };
 
 static guint _gaim_blist_hbuddy_hash (struct _gaim_hbuddy *hb)
@@ -78,7 +69,26 @@ static guint _gaim_blist_hbuddy_hash (struct _gaim_hbuddy *hb)
 
 static guint _gaim_blist_hbuddy_equal (struct _gaim_hbuddy *hb1, struct _gaim_hbuddy *hb2)
 {
-	return ((!strcmp(hb1->name, hb2->name)) && hb1->account == hb2->account);
+	return ((!strcmp(hb1->name, hb2->name)) && hb1->account == hb2->account && hb1->group == hb2->group);
+}
+
+static void blist_pref_cb(const char *name, GaimPrefType typ, gpointer value, gpointer data)
+{
+	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
+	GaimBlistNode *group, *buddy;
+
+	if (!ops)
+		return;
+
+	for(group = gaimbuddylist->root; group; group = group->next) {
+		if(!GAIM_BLIST_NODE_IS_GROUP(group))
+			continue;
+		for(buddy = group->child; buddy; buddy = buddy->next) {
+			if(!GAIM_BLIST_NODE_IS_BUDDY(buddy))
+				continue;
+			ops->update(gaimbuddylist, buddy);
+		}
+	}
 }
 
 /*****************************************************************************
@@ -91,12 +101,16 @@ struct gaim_buddy_list *gaim_blist_new()
 
 	gbl->ui_ops = gaim_get_blist_ui_ops();
 
-	gbl->buddies = g_hash_table_new ((GHashFunc)_gaim_blist_hbuddy_hash, 
+	gbl->buddies = g_hash_table_new ((GHashFunc)_gaim_blist_hbuddy_hash,
 					 (GEqualFunc)_gaim_blist_hbuddy_equal);
 
 	if (gbl->ui_ops != NULL && gbl->ui_ops->new_list != NULL)
 		gbl->ui_ops->new_list(gbl);
 
+	gaim_prefs_connect_callback("/core/buddies/use_server_alias",
+								blist_pref_cb, NULL);
+
+
 	return gbl;
 }
 
@@ -144,9 +158,9 @@ void  gaim_blist_update_buddy_status (struct buddy *buddy, int status)
 
 	if((status & UC_UNAVAILABLE) != (buddy->uc & UC_UNAVAILABLE)) {
 		if(status & UC_UNAVAILABLE)
-			gaim_event_broadcast(event_buddy_away, buddy->account->gc, buddy->name);
+			gaim_signal_emit(gaim_blist_get_handle(), "buddy-away", buddy);
 		else
-			gaim_event_broadcast(event_buddy_back, buddy->account->gc, buddy->name);
+			gaim_signal_emit(gaim_blist_get_handle(), "buddy-back", buddy);
 	}
 
 	buddy->uc = status;
@@ -156,10 +170,12 @@ void  gaim_blist_update_buddy_status (struct buddy *buddy, int status)
 
 static gboolean presence_update_timeout_cb(struct buddy *buddy) {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
+	GaimConversation *conv;
+
+	conv = gaim_find_conversation_with_account(buddy->name, buddy->account);
 
 	if(buddy->present == GAIM_BUDDY_SIGNING_ON) {
 		buddy->present = GAIM_BUDDY_ONLINE;
-		gaim_event_broadcast(event_buddy_signon, buddy->account->gc, buddy->name);
 	} else if(buddy->present == GAIM_BUDDY_SIGNING_OFF) {
 		buddy->present = GAIM_BUDDY_OFFLINE;
 	}
@@ -169,6 +185,13 @@ static gboolean presence_update_timeout_cb(struct buddy *buddy) {
 	if (ops)
 		ops->update(gaimbuddylist, (GaimBlistNode*)buddy);
 
+	if (conv) {
+		if (buddy->present == GAIM_BUDDY_ONLINE)
+			gaim_conversation_update(conv, GAIM_CONV_ACCOUNT_ONLINE);
+		else if (buddy->present == GAIM_BUDDY_OFFLINE)
+			gaim_conversation_update(conv, GAIM_CONV_ACCOUNT_OFFLINE);
+	}
+
 	return FALSE;
 }
 
@@ -178,12 +201,12 @@ void gaim_blist_update_buddy_presence(struct buddy *buddy, int presence) {
 
 	if (!GAIM_BUDDY_IS_ONLINE(buddy) && presence) {
 		buddy->present = GAIM_BUDDY_SIGNING_ON;
-		gaim_event_broadcast(event_buddy_signon, buddy->account->gc, buddy->name);
+		gaim_signal_emit(gaim_blist_get_handle(), "buddy-signed-on", buddy);
 		do_timer = TRUE;
 		((struct group *)((GaimBlistNode *)buddy)->parent)->online++;
 	} else if(GAIM_BUDDY_IS_ONLINE(buddy) && !presence) {
 		buddy->present = GAIM_BUDDY_SIGNING_OFF;
-		gaim_event_broadcast(event_buddy_signoff, buddy->account->gc, buddy->name);
+		gaim_signal_emit(gaim_blist_get_handle(), "buddy-signed-off", buddy);
 		do_timer = TRUE;
 		((struct group *)((GaimBlistNode *)buddy)->parent)->online--;
 	}
@@ -221,7 +244,7 @@ void gaim_blist_update_buddy_icon(struct buddy *buddy) {
 void  gaim_blist_rename_buddy (struct buddy *buddy, const char *name)
 {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
-       	g_free(buddy->name);
+	g_free(buddy->name);
 	buddy->name = g_strdup(name);
 	if (ops)
 		ops->update(gaimbuddylist, (GaimBlistNode*)buddy);
@@ -245,7 +268,7 @@ void gaim_blist_alias_chat(struct chat *chat, const char *alias)
 void  gaim_blist_alias_buddy (struct buddy *buddy, const char *alias)
 {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
-	struct gaim_conversation *conv;
+	GaimConversation *conv;
 
 	g_free(buddy->alias);
 
@@ -263,6 +286,27 @@ void  gaim_blist_alias_buddy (struct buddy *buddy, const char *alias)
 		gaim_conversation_autoset_title(conv);
 }
 
+void  gaim_blist_server_alias_buddy (struct buddy *buddy, const char *alias)
+{
+	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
+	GaimConversation *conv;
+
+	g_free(buddy->server_alias);
+
+	if(alias && strlen(alias) && g_utf8_validate(alias, -1, NULL))
+		buddy->server_alias = g_strdup(alias);
+	else
+		buddy->server_alias = NULL;
+
+	if (ops)
+		ops->update(gaimbuddylist, (GaimBlistNode*)buddy);
+
+	conv = gaim_find_conversation_with_account(buddy->name, buddy->account);
+
+	if (conv)
+		gaim_conversation_autoset_title(conv);
+}
+
 void gaim_blist_rename_group(struct group *group, const char *name)
 {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
@@ -294,14 +338,14 @@ void gaim_blist_rename_group(struct group *group, const char *name)
 			child = next;
 		}
 		for (accts = gaim_group_get_accounts(group); accts; accts = g_slist_remove(accts, accts->data)) {
-			struct gaim_account *account = accts->data;
+			GaimAccount *account = accts->data;
 			serv_rename_group(account->gc, group, name);
 		}
 		gaim_blist_remove_group(group);
 	} else {
 		/* a simple rename */
 		for (accts = gaim_group_get_accounts(group); accts; accts = g_slist_remove(accts, accts->data)) {
-			struct gaim_account *account = accts->data;
+			GaimAccount *account = accts->data;
 			serv_rename_group(account->gc, group, name);
 		}
 		g_free(group->name);
@@ -311,7 +355,7 @@ void gaim_blist_rename_group(struct group *group, const char *name)
 	}
 }
 
-struct chat *gaim_chat_new(struct gaim_account *account, const char *alias, GHashTable *components)
+struct chat *gaim_chat_new(GaimAccount *account, const char *alias, GHashTable *components)
 {
 	struct chat *chat;
 	struct gaim_blist_ui_ops *ops;
@@ -324,6 +368,8 @@ struct chat *gaim_chat_new(struct gaim_account *account, const char *alias, GHas
 	if(alias && strlen(alias))
 		chat->alias = g_strdup(alias);
 	chat->components = components;
+	chat->settings = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, g_free);
 
 	((GaimBlistNode*)chat)->type = GAIM_BLIST_CHAT_NODE;
 
@@ -335,7 +381,34 @@ struct chat *gaim_chat_new(struct gaim_account *account, const char *alias, GHas
 	return chat;
 }
 
-struct buddy *gaim_buddy_new(struct gaim_account *account, const char *screenname, const char *alias)
+char *gaim_chat_get_display_name(struct chat *chat)
+{
+	char *name;
+
+	if(chat->alias){
+		 name = g_strdup(chat->alias);
+	}
+	else{
+		 GList *parts;
+		 GaimPlugin *prpl;
+		 GaimPluginProtocolInfo *prpl_info;
+		 struct proto_chat_entry *pce;
+
+		 prpl = gaim_find_prpl(gaim_account_get_protocol(chat->account));
+		 prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(prpl);
+
+		 parts = prpl_info->chat_info(chat->account->gc);
+
+		 pce = parts->data;
+		 name = g_markup_escape_text(g_hash_table_lookup(chat->components,
+														 pce->identifier), -1);
+		 g_list_free(parts);
+	}
+
+	return name;
+}
+
+struct buddy *gaim_buddy_new(GaimAccount *account, const char *screenname, const char *alias)
 {
 	struct buddy *b;
 	struct gaim_blist_ui_ops *ops;
@@ -354,6 +427,7 @@ struct buddy *gaim_buddy_new(struct gaim_account *account, const char *screennam
 
 	return b;
 }
+
 void gaim_blist_add_chat(struct chat *chat, struct group *group, GaimBlistNode *node)
 {
 	GaimBlistNode *n = node, *cnode = (GaimBlistNode*)chat;
@@ -364,9 +438,9 @@ void gaim_blist_add_chat(struct chat *chat, struct group *group, GaimBlistNode *
 	if (!n) {
 		if (!g) {
 			g = gaim_group_new(_("Chats"));
-			gaim_blist_add_group(g, NULL);
+			gaim_blist_add_group(g,
+					gaim_blist_get_last_sibling(gaimbuddylist->root));
 		}
-		n = gaim_blist_get_last_child((GaimBlistNode*)g);
 	} else {
 		g = (struct group*)n->parent;
 	}
@@ -380,7 +454,7 @@ void gaim_blist_add_chat(struct chat *chat, struct group *group, GaimBlistNode *
 		 * being moved.
 		 */
 		((struct group *)cnode->parent)->totalsize--;
-		if (chat->account->gc) {
+		if (gaim_account_is_connected(chat->account)) {
 			((struct group *)cnode->parent)->online--;
 			((struct group *)cnode->parent)->currentsize--;
 		}
@@ -404,16 +478,19 @@ void gaim_blist_add_chat(struct chat *chat, struct group *group, GaimBlistNode *
 		cnode->parent = n->parent;
 		n->next = cnode;
 		((struct group *)n->parent)->totalsize++;
-		if (chat->account->gc) {
+		if (gaim_account_is_connected(chat->account)) {
 			((struct group *)n->parent)->online++;
 			((struct group *)n->parent)->currentsize++;
 		}
 	} else {
+		if(((GaimBlistNode*)g)->child)
+			((GaimBlistNode*)g)->child->prev = cnode;
+		cnode->next = ((GaimBlistNode*)g)->child;
+		cnode->prev = NULL;
 		((GaimBlistNode*)g)->child = cnode;
-		cnode->next = cnode->prev = NULL;
 		cnode->parent = (GaimBlistNode*)g;
 		g->totalsize++;
-		if (chat->account->gc) {
+		if (gaim_account_is_connected(chat->account)) {
 			g->online++;
 			g->currentsize++;
 		}
@@ -436,9 +513,9 @@ void  gaim_blist_add_buddy (struct buddy *buddy, struct group *group, GaimBlistN
 	if (!n) {
 		if (!g) {
 			g = gaim_group_new(_("Buddies"));
-			gaim_blist_add_group(g, NULL);
+			gaim_blist_add_group(g,
+					gaim_blist_get_last_sibling(gaimbuddylist->root));
 		}
-		n = gaim_blist_get_last_child((GaimBlistNode*)g);
 	} else {
 		g = (struct group*)n->parent;
 	}
@@ -452,7 +529,7 @@ void  gaim_blist_add_buddy (struct buddy *buddy, struct group *group, GaimBlistN
 		 * being moved.
 		 */
 		((struct group *)bnode->parent)->totalsize--;
-		if (buddy->account->gc)
+		if (gaim_account_is_connected(buddy->account))
 			((struct group *)bnode->parent)->currentsize--;
 		if (GAIM_BUDDY_IS_ONLINE(buddy))
 			((struct group *)bnode->parent)->online--;
@@ -480,17 +557,19 @@ void  gaim_blist_add_buddy (struct buddy *buddy, struct group *group, GaimBlistN
 		((GaimBlistNode*)buddy)->parent = n->parent;
 		n->next = (GaimBlistNode*)buddy;
 		((struct group *)n->parent)->totalsize++;
-		if (buddy->account->gc)
+		if (gaim_account_is_connected(buddy->account))
 			((struct group *)n->parent)->currentsize++;
 		if (GAIM_BUDDY_IS_ONLINE(buddy))
 			((struct group *)n->parent)->online++;
 	} else {
-		((GaimBlistNode*)g)->child = (GaimBlistNode*)buddy;
-		((GaimBlistNode*)buddy)->next = NULL;
+		if(((GaimBlistNode*)g)->child)
+			((GaimBlistNode*)g)->child->prev = (GaimBlistNode*)buddy;
 		((GaimBlistNode*)buddy)->prev = NULL;
+		((GaimBlistNode*)buddy)->next = ((GaimBlistNode*)g)->child;
+		((GaimBlistNode*)g)->child = (GaimBlistNode*)buddy;
 		((GaimBlistNode*)buddy)->parent = (GaimBlistNode*)g;
 		g->totalsize++;
-		if (buddy->account->gc)
+		if (gaim_account_is_connected(buddy->account))
 			g->currentsize++;
 		if (GAIM_BUDDY_IS_ONLINE(buddy))
 			g->online++;
@@ -499,6 +578,7 @@ void  gaim_blist_add_buddy (struct buddy *buddy, struct group *group, GaimBlistN
 	hb = g_malloc(sizeof(struct _gaim_hbuddy));
 	hb->name = g_strdup(normalize(buddy->name));
 	hb->account = buddy->account;
+	hb->group = ((GaimBlistNode*)buddy)->parent;
 
 	if (g_hash_table_lookup(gaimbuddylist->buddies, (gpointer)hb)) {
 		/* This guy already exists */
@@ -553,9 +633,6 @@ void  gaim_blist_add_group (struct group *group, GaimBlistNode *node)
 		return;
 	}
 
-	if (!node)
-		node = gaim_blist_get_last_sibling(gaimbuddylist->root);
-
 	/* if we're moving to overtop of ourselves, do nothing */
 	if(gnode == node)
 		return;
@@ -575,11 +652,19 @@ void  gaim_blist_add_group (struct group *group, GaimBlistNode *node)
 		save = TRUE;
 	}
 
-	gnode->next = node->next;
-	gnode->prev = node;
-	if(node->next)
-		node->next->prev = gnode;
-	node->next = gnode;
+	if (node) {
+		gnode->next = node->next;
+		gnode->prev = node;
+		if(node->next)
+			node->next->prev = gnode;
+		node->next = gnode;
+	} else {
+		gaimbuddylist->root->prev = gnode;
+		gnode->next = gaimbuddylist->root;
+		gnode->prev = NULL;
+		gaimbuddylist->root = gnode;
+	}
+
 
 	if (ops) {
 		ops->update(gaimbuddylist, gnode);
@@ -609,13 +694,14 @@ void  gaim_blist_remove_buddy (struct buddy *buddy)
 	if (node->next)
 		node->next->prev = node->prev;
 	group->totalsize--;
-	if (buddy->account->gc)
+	if (gaim_account_is_connected(buddy->account))
 		group->currentsize--;
 	if (GAIM_BUDDY_IS_ONLINE(buddy))
 		group->online--;
 
 	hb.name = normalize(buddy->name);
 	hb.account = buddy->account;
+	hb.group = ((GaimBlistNode*)buddy)->parent;
 	if (g_hash_table_lookup_extended(gaimbuddylist->buddies, &hb, (gpointer *)&key, (gpointer *)&val)) {
 		g_hash_table_remove(gaimbuddylist->buddies, &hb);
 		g_free(key->name);
@@ -649,7 +735,7 @@ void  gaim_blist_remove_chat (struct chat *chat)
 	if (node->next)
 		node->next->prev = node->prev;
 	group->totalsize--;
-	if (chat->account->gc) {
+	if (gaim_account_is_connected(chat->account)) {
 		group->currentsize--;
 		group->online--;
 	}
@@ -675,11 +761,15 @@ void  gaim_blist_remove_group (struct group *group)
 			child = child->next;
 		}
 
-		buf = g_strdup_printf(_("%d buddies from group %s were not "
-					"removed because their accounts were not logged in.  These "
-					"buddies and the group were not removed.\n"),
-				count, group->name);
-
+		buf = g_strdup_printf(ngettext("%d buddy from group %s was not removed "
+									   "because its account was not logged in."
+									   "  This buddy and the group were not "
+									   "removed.\n", 
+									   "%d buddies from group %s were not "
+									   "removed because their accounts were "
+									   "not logged in.  These buddies and "
+									   "the group were not removed.\n", count),
+							  count, group->name);
 		gaim_notify_error(NULL, NULL, _("Group not removed"), buf);
 		g_free(buf);
 		return;
@@ -698,37 +788,78 @@ void  gaim_blist_remove_group (struct group *group)
 }
 
 char *gaim_get_buddy_alias_only(struct buddy *b) {
-        if(!b)
-                return NULL;
-        if(b->alias && b->alias[0])
-                return b->alias;
-        else if((misc_options & OPT_MISC_USE_SERVER_ALIAS) && b->server_alias)
-                return b->server_alias;
-        return NULL;
+	if(!b)
+		return NULL;
+
+	if(b->alias && b->alias[0]) {
+		return b->alias;
+	}
+	else if (b->server_alias != NULL &&
+			 gaim_prefs_get_bool("/core/buddies/use_server_alias")) {
+
+		return b->server_alias;
+	}
+
+	return NULL;
 }
 
 char *  gaim_get_buddy_alias (struct buddy *buddy)
 {
 	char *ret = gaim_get_buddy_alias_only(buddy);
-        if(!ret)
-                return buddy ? buddy->name : _("Unknown");
-        return ret;
 
+	if(!ret)
+		return buddy ? buddy->name : _("Unknown");
+
+	return ret;
 }
 
-struct buddy *gaim_find_buddy(struct gaim_account *account, const char *name)
+struct buddy *gaim_find_buddy(GaimAccount *account, const char *name)
 {
 	struct buddy *buddy;
 	struct _gaim_hbuddy hb;
+	GaimBlistNode *group;
 
 	if (!gaimbuddylist)
 		return NULL;
 
+	if (!name)
+		return NULL;
+
 	hb.name = normalize(name);
 	hb.account = account;
-	buddy = g_hash_table_lookup(gaimbuddylist->buddies, &hb);
 
-	return buddy;
+	for(group = gaimbuddylist->root; group; group = group->next) {
+		hb.group = group;
+		if ((buddy = g_hash_table_lookup(gaimbuddylist->buddies, &hb)) != NULL)
+			return buddy;
+	}
+
+	return NULL;
+}
+
+GSList *gaim_find_buddies(GaimAccount *account, const char *name)
+{
+	struct buddy *buddy;
+	struct _gaim_hbuddy hb;
+	GaimBlistNode *group;
+	GSList *ret = NULL;
+
+	if (!gaimbuddylist)
+		return NULL;
+
+	if (!name)
+		return NULL;
+
+	hb.name = normalize(name);
+	hb.account = account;
+
+	for(group = gaimbuddylist->root; group; group = group->next) {
+		hb.group = group;
+		if ((buddy = g_hash_table_lookup(gaimbuddylist->buddies, &hb)) != NULL)
+			ret = g_slist_append(ret, buddy);
+	}
+
+	return ret;
 }
 
 struct group *gaim_find_group(const char *name)
@@ -744,6 +875,57 @@ struct group *gaim_find_group(const char *name)
 	}
 	return NULL;
 }
+
+struct chat *
+gaim_blist_find_chat(GaimAccount *account, const char *name)
+{
+	char *chat_name;
+	struct chat *chat;
+	GaimPlugin *prpl;
+	GaimPluginProtocolInfo *prpl_info = NULL;
+	struct proto_chat_entry *pce;
+	GaimBlistNode *node, *group;
+	GList *parts;
+
+	g_return_val_if_fail(gaim_get_blist() != NULL, NULL);
+	g_return_val_if_fail(name != NULL, NULL);
+
+	for (group = gaimbuddylist->root; group != NULL; group = group->next) {
+		for (node = group->child; node != NULL; node = node->next) {
+			if (GAIM_BLIST_NODE_IS_CHAT(node)) {
+
+				chat = (struct chat *)node;
+
+				prpl = gaim_find_prpl(gaim_account_get_protocol(chat->account));
+				prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(prpl);
+
+				parts = prpl_info->chat_info(
+					gaim_account_get_connection(chat->account));
+
+				pce = parts->data;
+				chat_name = g_hash_table_lookup(chat->components,
+												pce->identifier);
+
+				if (chat->account == account &&
+					name != NULL && !strcmp(chat_name, name)) {
+
+					return chat;
+				}
+			}
+		}
+	}
+
+	return NULL;
+}
+
+struct group *
+gaim_blist_chat_get_group(struct chat *chat)
+{
+	g_return_val_if_fail(chat != NULL, NULL);
+
+	return (struct group *)(((GaimBlistNode *)chat)->parent);
+}
+
 struct group *gaim_find_buddys_group(struct buddy *buddy)
 {
 	if (!buddy)
@@ -757,14 +939,19 @@ GSList *gaim_group_get_accounts(struct group *g)
 	GaimBlistNode *child = ((GaimBlistNode *)g)->child;
 
 	while (child) {
-		if (!g_slist_find(l, ((struct buddy*)child)->account))
-			l = g_slist_append(l, ((struct buddy*)child)->account);
+		GaimAccount *account = NULL;
+		if (GAIM_BLIST_NODE_IS_BUDDY(child))
+			account = ((struct buddy *)child)->account;
+		else if (GAIM_BLIST_NODE_IS_CHAT(child))
+			account = ((struct chat *)child)->account;
+		if (!g_slist_find(l, account))
+			l = g_slist_append(l, account);
 		child = child->next;
 	}
 	return l;
 }
 
-void gaim_blist_add_account(struct gaim_account *account)
+void gaim_blist_add_account(GaimAccount *account)
 {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
 	GaimBlistNode *group, *buddy;
@@ -794,7 +981,7 @@ void gaim_blist_add_account(struct gaim_account *account)
 	}
 }
 
-void gaim_blist_remove_account(struct gaim_account *account)
+void gaim_blist_remove_account(GaimAccount *account)
 {
 	struct gaim_blist_ui_ops *ops = gaimbuddylist->ui_ops;
 	GaimBlistNode *group, *buddy;
@@ -827,7 +1014,7 @@ void gaim_blist_remove_account(struct gaim_account *account)
 	}
 }
 
-void parse_toc_buddy_list(struct gaim_account *account, char *config)
+void parse_toc_buddy_list(GaimAccount *account, char *config)
 {
 	char *c;
 	char current[256];
@@ -854,7 +1041,8 @@ void parse_toc_buddy_list(struct gaim_account *account, char *config)
 				}
 				if (!gaim_find_group(current)) {
 					struct group *g = gaim_group_new(current);
-					gaim_blist_add_group(g, NULL);
+					gaim_blist_add_group(g,
+							gaim_blist_get_last_sibling(gaimbuddylist->root));
 				}
 			} else if (*c == 'b') { /*&& !gaim_find_buddy(user, c + 2)) {*/
 				char nm[80], sw[388], *a, *utf8 = NULL;
@@ -884,25 +1072,26 @@ void parse_toc_buddy_list(struct gaim_account *account, char *config)
 				if (!gaim_find_buddy(account, nm)) {
 					struct buddy *b = gaim_buddy_new(account, nm, sw);
 					struct group *g = gaim_find_group(current);
-					gaim_blist_add_buddy(b, g, NULL);
+					gaim_blist_add_buddy(b, g,
+							gaim_blist_get_last_child((GaimBlistNode*)g));
 					bud = g_list_append(bud, g_strdup(nm));
 				}
 			} else if (*c == 'p') {
-				gaim_privacy_permit_add(account, c + 2);
+				gaim_privacy_permit_add(account, c + 2, TRUE);
 			} else if (*c == 'd') {
-				gaim_privacy_deny_add(account, c + 2);
+				gaim_privacy_deny_add(account, c + 2, TRUE);
 			} else if (!strncmp("toc", c, 3)) {
-				sscanf(c + strlen(c) - 1, "%d", &account->permdeny);
+				sscanf(c + strlen(c) - 1, "%d", &account->perm_deny);
 				gaim_debug(GAIM_DEBUG_MISC, "toc blist",
-						   "permdeny: %d\n", account->permdeny);
-				if (account->permdeny == 0)
-					account->permdeny = 1;
+						   "permdeny: %d\n", account->perm_deny);
+				if (account->perm_deny == 0)
+					account->perm_deny = 1;
 			} else if (*c == 'm') {
-				sscanf(c + 2, "%d", &account->permdeny);
+				sscanf(c + 2, "%d", &account->perm_deny);
 				gaim_debug(GAIM_DEBUG_MISC, "toc blist",
-						   "permdeny: %d\n", account->permdeny);
-				if (account->permdeny == 0)
-					account->permdeny = 1;
+						   "permdeny: %d\n", account->perm_deny);
+				if (account->perm_deny == 0)
+					account->perm_deny = 1;
 			}
 		} while ((c = strtok(NULL, "\n")));
 
@@ -1098,7 +1287,7 @@ static gchar *get_screenname_filename(const char *name)
 static gboolean gaim_blist_read(const char *filename);
 
 
-static void do_import(struct gaim_account *account, const char *filename)
+static void do_import(GaimAccount *account, const char *filename)
 {
 	GString *buf = NULL;
 	char first[64];
@@ -1111,8 +1300,22 @@ static void do_import(struct gaim_account *account, const char *filename)
 		g_snprintf(path, sizeof(path), "%s", filename);
 	} else {
 		char *g_screenname = get_screenname_filename(account->username);
+		const char *username;
 		char *file = gaim_user_dir();
-		int protocol = (account->protocol == GAIM_PROTO_OSCAR) ? (isalpha(account->username[0]) ? GAIM_PROTO_TOC : GAIM_PROTO_ICQ): account->protocol;
+		GaimProtocol prpl_num;
+		int protocol;
+		
+		prpl_num = gaim_account_get_protocol(account);
+
+		protocol = prpl_num;
+
+		/* TODO Somehow move this checking into prpls */
+		if (prpl_num == GAIM_PROTO_OSCAR) {
+			if ((username = gaim_account_get_username(account)) != NULL) {
+				protocol = (isalpha(*username)
+							? GAIM_PROTO_TOC : GAIM_PROTO_ICQ);
+			}
+		}
 
 		if (file != (char *)NULL) {
 			snprintf(path, PATHSIZE, "%s" G_DIR_SEPARATOR_S "%s.%d.blist", file, g_screenname, protocol);
@@ -1191,13 +1394,14 @@ static void do_import(struct gaim_account *account, const char *filename)
 	}
 }
 
-gboolean gaim_group_on_account(struct group *g, struct gaim_account *account) {
+gboolean gaim_group_on_account(struct group *g, GaimAccount *account) {
 	GaimBlistNode *bnode;
 	for(bnode = g->node.child; bnode; bnode = bnode->next) {
 		struct buddy *b = (struct buddy *)bnode;
 		if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
 			continue;
-		if((!account && b->account->gc) || b->account == account)
+		if((!account && gaim_account_is_connected(b->account))
+				|| b->account == account)
 			return TRUE;
 	}
 	return FALSE;
@@ -1217,6 +1421,7 @@ static char *blist_parser_buddy_alias = NULL;
 static char *blist_parser_setting_name = NULL;
 static char *blist_parser_setting_value = NULL;
 static GHashTable *blist_parser_buddy_settings = NULL;
+static GHashTable *blist_parser_chat_settings = NULL;
 static GHashTable *blist_parser_group_settings = NULL;
 static GHashTable *blist_parser_chat_components = NULL;
 static int blist_parser_privacy_mode = 0;
@@ -1262,7 +1467,8 @@ static void blist_start_element_handler (GMarkupParseContext *context,
 		}
 		if(blist_parser_group_name) {
 			struct group *g = gaim_group_new(blist_parser_group_name);
-			gaim_blist_add_group(g,NULL);
+			gaim_blist_add_group(g,
+					gaim_blist_get_last_sibling(gaimbuddylist->root));
 		}
 	} else if(!strcmp(element_name, "chat")) {
 		tag_stack = g_list_prepend(tag_stack, GINT_TO_POINTER(BLIST_TAG_CHAT));
@@ -1351,30 +1557,37 @@ static void blist_end_element_handler(GMarkupParseContext *context,
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 		blist_parser_group_settings = NULL;
 	} else if(!strcmp(element_name, "chat")) {
-		struct gaim_account *account = gaim_account_find(blist_parser_account_name,
+		GaimAccount *account = gaim_accounts_find(blist_parser_account_name,
 				blist_parser_account_protocol);
 		if(account) {
 			struct chat *chat = gaim_chat_new(account, blist_parser_chat_alias, blist_parser_chat_components);
 			struct group *g = gaim_find_group(blist_parser_group_name);
-			gaim_blist_add_chat(chat,g,NULL);
+			gaim_blist_add_chat(chat,g,
+					gaim_blist_get_last_child((GaimBlistNode*)g));
+			if(blist_parser_chat_settings) {
+				g_hash_table_destroy(chat->settings);
+				chat->settings = blist_parser_chat_settings;
+			}
 		}
 		g_free(blist_parser_chat_alias);
 		blist_parser_chat_alias = NULL;
 		g_free(blist_parser_account_name);
 		blist_parser_account_name = NULL;
 		blist_parser_chat_components = NULL;
+		blist_parser_chat_settings = NULL;
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 	} else if(!strcmp(element_name, "person")) {
 		g_free(blist_parser_person_name);
 		blist_parser_person_name = NULL;
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 	} else if(!strcmp(element_name, "buddy")) {
-		struct gaim_account *account = gaim_account_find(blist_parser_account_name,
+		GaimAccount *account = gaim_accounts_find(blist_parser_account_name,
 				blist_parser_account_protocol);
 		if(account) {
 			struct buddy *b = gaim_buddy_new(account, blist_parser_buddy_name, blist_parser_buddy_alias);
 			struct group *g = gaim_find_group(blist_parser_group_name);
-			gaim_blist_add_buddy(b,g,NULL);
+			gaim_blist_add_buddy(b,g,
+					gaim_blist_get_last_child((GaimBlistNode*)g));
 			if(blist_parser_buddy_settings) {
 				g_hash_table_destroy(b->settings);
 				b->settings = blist_parser_buddy_settings;
@@ -1416,6 +1629,15 @@ static void blist_end_element_handler(GMarkupParseContext *context,
 						g_strdup(blist_parser_setting_name),
 						g_strdup(blist_parser_setting_value));
 			}
+		} else if(GPOINTER_TO_INT(tag_stack->next->data) == BLIST_TAG_CHAT) {
+			if(!blist_parser_chat_settings)
+				blist_parser_chat_settings = g_hash_table_new_full(g_str_hash,
+						g_str_equal, g_free, g_free);
+			if(blist_parser_setting_name && blist_parser_setting_value) {
+				g_hash_table_replace(blist_parser_chat_settings,
+						g_strdup(blist_parser_setting_name),
+						g_strdup(blist_parser_setting_value));
+			}
 		} else if(GPOINTER_TO_INT(tag_stack->next->data) == BLIST_TAG_GROUP) {
 			if(!blist_parser_group_settings)
 				blist_parser_group_settings = g_hash_table_new_full(g_str_hash,
@@ -1434,28 +1656,28 @@ static void blist_end_element_handler(GMarkupParseContext *context,
 	} else if(!strcmp(element_name, "privacy")) {
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 	} else if(!strcmp(element_name, "account")) {
-		struct gaim_account *account = gaim_account_find(blist_parser_account_name,
+		GaimAccount *account = gaim_accounts_find(blist_parser_account_name,
 				blist_parser_account_protocol);
 		if(account) {
-			account->permdeny = blist_parser_privacy_mode;
+			account->perm_deny = blist_parser_privacy_mode;
 		}
 		g_free(blist_parser_account_name);
 		blist_parser_account_name = NULL;
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 	} else if(!strcmp(element_name, "permit")) {
-		struct gaim_account *account = gaim_account_find(blist_parser_account_name,
+		GaimAccount *account = gaim_accounts_find(blist_parser_account_name,
 				blist_parser_account_protocol);
 		if(account) {
-			gaim_privacy_permit_add(account, blist_parser_buddy_name);
+			gaim_privacy_permit_add(account, blist_parser_buddy_name, TRUE);
 		}
 		g_free(blist_parser_buddy_name);
 		blist_parser_buddy_name = NULL;
 		tag_stack = g_list_delete_link(tag_stack, tag_stack);
 	} else if(!strcmp(element_name, "block")) {
-		struct gaim_account *account = gaim_account_find(blist_parser_account_name,
+		GaimAccount *account = gaim_accounts_find(blist_parser_account_name,
 				blist_parser_account_protocol);
 		if(account) {
-			gaim_privacy_deny_add(account, blist_parser_buddy_name);
+			gaim_privacy_deny_add(account, blist_parser_buddy_name, TRUE);
 		}
 		g_free(blist_parser_buddy_name);
 		blist_parser_buddy_name = NULL;
@@ -1555,7 +1777,7 @@ static gboolean gaim_blist_read(const char *filename) {
 }
 
 void gaim_blist_load() {
-	GSList *accts;
+	GList *accts;
 	char *user_dir = gaim_user_dir();
 	char *filename;
 	char *msg;
@@ -1574,7 +1796,10 @@ void gaim_blist_load() {
 			gaim_notify_error(NULL, NULL, _("Buddy List Error"), msg);
 			g_free(msg);
 		}
-	} else if(g_slist_length(gaim_accounts)) {
+	} else if(g_list_length(gaim_accounts_get_all())) {
+#if 0
+		GMainContext *ctx;
+
 		/* rob wants to inform the user that their buddy lists are
 		 * being converted */
 		msg = g_strdup_printf(_("Gaim is converting your old buddy lists "
@@ -1584,11 +1809,14 @@ void gaim_blist_load() {
 		g_free(msg);
 
 		/* now, let gtk actually display the dialog before we start anything */
-		while(gtk_events_pending())
-			gtk_main_iteration();
+		ctx = g_main_context_default();
+
+		while(g_main_context_pending(ctx))
+			g_main_context_iteration(ctx, FALSE);
+#endif
 
 		/* read in the old lists, then save to the new format */
-		for(accts = gaim_accounts; accts; accts = accts->next) {
+		for(accts = gaim_accounts_get_all(); accts; accts = accts->next) {
 			do_import(accts->data, NULL);
 		}
 		gaim_blist_save();
@@ -1651,8 +1879,9 @@ static void blist_print_chat_components(gpointer key, gpointer data,
 	g_free(data_val);
 }
 
-static void gaim_blist_write(FILE *file, struct gaim_account *exp_acct) {
-	GSList *accounts, *buds;
+static void gaim_blist_write(FILE *file, GaimAccount *exp_acct) {
+	GList *accounts;
+	GSList *buds;
 	GaimBlistNode *gnode,*bnode;
 	struct group *group;
 	struct buddy *bud;
@@ -1680,7 +1909,8 @@ static void gaim_blist_write(FILE *file, struct gaim_account *exp_acct) {
 						fprintf(file, "\t\t\t<person name=\"%s\">\n",
 								bud_alias ? bud_alias : bud_name);
 						fprintf(file, "\t\t\t\t<buddy protocol=\"%d\" "
-								"account=\"%s\">\n", bud->account->protocol,
+								"account=\"%s\">\n",
+								gaim_account_get_protocol(bud->account),
 								acct_name);
 						fprintf(file, "\t\t\t\t\t<name>%s</name>\n", bud_name);
 						if(bud_alias) {
@@ -1699,7 +1929,9 @@ static void gaim_blist_write(FILE *file, struct gaim_account *exp_acct) {
 					struct chat *chat = (struct chat *)bnode;
 					if(!exp_acct || chat->account == exp_acct) {
 						char *acct_name = g_markup_escape_text(chat->account->username, -1);
-						fprintf(file, "\t\t\t<chat protocol=\"%d\" account=\"%s\">\n", chat->account->protocol, acct_name);
+						fprintf(file, "\t\t\t<chat protocol=\"%d\" account=\"%s\">\n",
+								gaim_account_get_protocol(chat->account),
+								acct_name);
 						if(chat->alias) {
 							char *chat_alias = g_markup_escape_text(chat->alias, -1);
 							fprintf(file, "\t\t\t\t<alias>%s</alias>\n", chat_alias);
@@ -1707,6 +1939,9 @@ static void gaim_blist_write(FILE *file, struct gaim_account *exp_acct) {
 						}
 						g_hash_table_foreach(chat->components,
 								blist_print_chat_components, file);
+						/* works for chats too, I don't feel like renaming */
+						g_hash_table_foreach(chat->settings,
+								blist_print_buddy_settings, file);
 						fprintf(file, "\t\t\t</chat>\n");
 						g_free(acct_name);
 					}
@@ -1720,12 +1955,16 @@ static void gaim_blist_write(FILE *file, struct gaim_account *exp_acct) {
 	fprintf(file, "\t</blist>\n");
 	fprintf(file, "\t<privacy>\n");
 
-	for(accounts = gaim_accounts; accounts; accounts = accounts->next) {
-		struct gaim_account *account = accounts->data;
+	for(accounts = gaim_accounts_get_all();
+		accounts != NULL;
+		accounts = accounts->next) {
+
+		GaimAccount *account = accounts->data;
 		char *acct_name = g_markup_escape_text(account->username, -1);
 		if(!exp_acct || account == exp_acct) {
 			fprintf(file, "\t\t<account protocol=\"%d\" name=\"%s\" "
-					"mode=\"%d\">\n", account->protocol, acct_name, account->permdeny);
+					"mode=\"%d\">\n", gaim_account_get_protocol(account),
+					acct_name, account->perm_deny);
 			for(buds = account->permit; buds; buds = buds->next) {
 				char *bud_name = g_markup_escape_text(buds->data, -1);
 				fprintf(file, "\t\t\t<permit>%s</permit>\n", bud_name);
@@ -1787,74 +2026,6 @@ void gaim_blist_save() {
 	g_free(filename_real);
 }
 
-gboolean gaim_privacy_permit_add(struct gaim_account *account, const char *who) {
-	GSList *d = account->permit;
-	char *n = g_strdup(normalize(who));
-	while(d) {
-		if(!gaim_utf8_strcasecmp(n, normalize(d->data)))
-			break;
-		d = d->next;
-	}
-	g_free(n);
-	if(!d) {
-		account->permit = g_slist_append(account->permit, g_strdup(who));
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-gboolean gaim_privacy_permit_remove(struct gaim_account *account, const char *who) {
-	GSList *d = account->permit;
-	char *n = g_strdup(normalize(who));
-	while(d) {
-		if(!gaim_utf8_strcasecmp(n, normalize(d->data)))
-			break;
-		d = d->next;
-	}
-	g_free(n);
-	if(d) {
-		account->permit = g_slist_remove(account->permit, d->data);
-		g_free(d->data);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-gboolean gaim_privacy_deny_add(struct gaim_account *account, const char *who) {
-	GSList *d = account->deny;
-	char *n = g_strdup(normalize(who));
-	while(d) {
-		if(!gaim_utf8_strcasecmp(n, normalize(d->data)))
-			break;
-		d = d->next;
-	}
-	g_free(n);
-	if(!d) {
-		account->deny = g_slist_append(account->deny, g_strdup(who));
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-gboolean gaim_privacy_deny_remove(struct gaim_account *account, const char *who) {
-	GSList *d = account->deny;
-	char *n = g_strdup(normalize(who));
-	while(d) {
-		if(!gaim_utf8_strcasecmp(n, normalize(d->data)))
-			break;
-		d = d->next;
-	}
-	g_free(n);
-	if(d) {
-		account->deny = g_slist_remove(account->deny, d->data);
-		g_free(d->data);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 void gaim_group_set_setting(struct group *g, const char *key,
 		const char *value) {
 	if(!g)
@@ -1868,6 +2039,21 @@ char *gaim_group_get_setting(struct group *g, const char *key) {
 	return g_strdup(g_hash_table_lookup(g->settings, key));
 }
 
+void gaim_chat_set_setting(struct chat *c, const char *key,
+		const char *value)
+{
+	if(!c)
+		return;
+	g_hash_table_replace(c->settings, g_strdup(key), g_strdup(value));
+}
+
+char *gaim_chat_get_setting(struct chat *c, const char *key)
+{
+	if(!c)
+		return NULL;
+	return g_strdup(g_hash_table_lookup(c->settings, key));
+}
+
 void gaim_buddy_set_setting(struct buddy *b, const char *key,
 		const char *value) {
 	if(!b)
@@ -1906,4 +2092,53 @@ int gaim_blist_get_group_online_count(struct group *group) {
 	return group->online;
 }
 
+void *
+gaim_blist_get_handle(void)
+{
+	static int handle;
 
+	return &handle;
+}
+
+void
+gaim_blist_init(void)
+{
+	void *handle = gaim_blist_get_handle();
+
+	gaim_signal_register(handle, "buddy-away",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+
+	gaim_signal_register(handle, "buddy-back",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+
+	gaim_signal_register(handle, "buddy-idle",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+	gaim_signal_register(handle, "buddy-unidle",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+
+	gaim_signal_register(handle, "buddy-signed-on",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+
+	gaim_signal_register(handle, "buddy-signed-off",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_BLIST_BUDDY));
+
+	gaim_signal_register(handle, "update-idle", gaim_marshal_VOID, NULL, 0);
+}
+
+void
+gaim_blist_uninit(void)
+{
+	gaim_signals_unregister_by_instance(gaim_blist_get_handle());
+}
