@@ -1,10 +1,10 @@
 /**
- * @file gtkdebug.c GTK+ Debug API
+ * @file gtkdebug.h GTK+ Debug API
  * @ingroup gtkui
  *
  * gaim
  *
- * Copyright (C) 2002-2003, Christian Hammond <chipx86@gnupdate.org>
+ * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,11 @@
 
 #include "debug.h"
 
+/**
+ * Initializes the GTK+ debug system.
+ */
+void gaim_gtk_debug_init(void);
+
 /**
  * Shows the debug window.
  */
