@@ -58,38 +58,12 @@
 
 #define TOOLTIP_TIMEOUT 500
 
-static gboolean gtk_motion_event_notify(GtkWidget *imhtml, GdkEventMotion *event, gpointer user_data);
-static gboolean gtk_leave_event_notify(GtkWidget *imhtml, GdkEventCrossing *event, gpointer user_data);
-
-static gboolean gtk_size_allocate_cb(GtkIMHtml *widget, GtkAllocation *alloc, gpointer user_data);
-
-static gboolean gaim_im_image_clicked(GtkWidget *w, GdkEvent *event, gaim_im_image *image);
-
-static gint gtk_imhtml_tip (gpointer data);
-
-
 /* POINT_SIZE converts from AIM font sizes to point sizes.  It probably should be redone in such a
  * way that it base the sizes off the default font size rather than using arbitrary font sizes. */
 #define MAX_FONT_SIZE 7
 #define POINT_SIZE(x) (options & GTK_IMHTML_USE_POINTSIZE ? x : _point_sizes [MIN ((x), MAX_FONT_SIZE) - 1])
 static gint _point_sizes [] = { 8, 10, 12, 14, 20, 30, 40 };
 
-/* The four elements present in a <FONT> tag contained in a struct */
-typedef struct _FontDetail   FontDetail;
-struct _FontDetail {
-	gushort size;
-	gchar *face;
-	gchar *fore;
-	gchar *back;
-	gchar *sml;
-};
-
-struct _GtkSmileyTree {
-	GString *values;
-	GtkSmileyTree **children;
-	GtkIMHtmlSmiley *image;
-};
-
 static GtkSmileyTree*
 gtk_smiley_tree_new ()
 {
@@ -120,7 +94,7 @@ gtk_smiley_tree_insert (GtkSmileyTree *tree,
 			t->children = g_realloc (t->children, t->values->len * sizeof (GtkSmileyTree *));
 			t->children [index] = g_new0 (GtkSmileyTree, 1);
 		} else
-			index = (int) pos - (int) t->values->str;
+			index = (int) pos - GPOINTER_TO_INT(t->values->str);
 		
 		t = t->children [index];
 		
@@ -149,10 +123,218 @@ void gtk_smiley_tree_destroy (GtkSmileyTree *tree)
 	}
 }
 
+static gboolean gtk_size_allocate_cb(GtkIMHtml *widget, GtkAllocation *alloc, gpointer user_data)
+{
+	GdkRectangle rect;
+
+	gtk_text_view_get_visible_rect(GTK_TEXT_VIEW(widget), &rect);
+	if(widget->old_rect.width != rect.width || widget->old_rect.height != rect.height){
+		GList *iter = GTK_IMHTML(widget)->scalables;
+
+		while(iter){
+			GtkIMHtmlScalable *scale = GTK_IMHTML_SCALABLE(iter->data);
+			scale->scale(scale, rect.width, rect.height);
+
+			iter = iter->next;
+		}
+	}
+
+	widget->old_rect = rect;
+	return FALSE;
+}
+
+static gint
+gtk_imhtml_tip_paint (GtkIMHtml *imhtml)
+{
+	PangoLayout *layout;
+
+	g_return_val_if_fail(GTK_IS_IMHTML(imhtml), FALSE);
+
+	layout = gtk_widget_create_pango_layout(imhtml->tip_window, imhtml->tip);
+
+	gtk_paint_flat_box (imhtml->tip_window->style, imhtml->tip_window->window, 
+						GTK_STATE_NORMAL, GTK_SHADOW_OUT, NULL, imhtml->tip_window,
+						"tooltip", 0, 0, -1, -1);
+
+	gtk_paint_layout (imhtml->tip_window->style, imhtml->tip_window->window, GTK_STATE_NORMAL,
+					  FALSE, NULL, imhtml->tip_window, NULL, 4, 4, layout);
+
+	g_object_unref(layout);
+	return FALSE;
+}
+
+static gint
+gtk_imhtml_tip (gpointer data)
+{
+	GtkIMHtml *imhtml = data;
+	PangoFontMetrics *font;
+	PangoLayout *layout;
+
+	gint gap, x, y, h, w, scr_w, baseline_skip;
+
+	g_return_val_if_fail(GTK_IS_IMHTML(imhtml), FALSE);
+
+	if (!imhtml->tip || !GTK_WIDGET_DRAWABLE (GTK_WIDGET(imhtml))) {
+		imhtml->tip_timer = 0;
+		return FALSE;
+	}
+	
+	if (imhtml->tip_window){
+		gtk_widget_destroy (imhtml->tip_window);
+		imhtml->tip_window = NULL;
+	}
+
+	imhtml->tip_timer = 0;
+	imhtml->tip_window = gtk_window_new (GTK_WINDOW_POPUP);
+	gtk_widget_set_app_paintable (imhtml->tip_window, TRUE);
+	gtk_window_set_resizable (GTK_WINDOW (imhtml->tip_window), FALSE);
+	gtk_widget_set_name (imhtml->tip_window, "gtk-tooltips");
+	g_signal_connect_swapped (G_OBJECT (imhtml->tip_window), "expose_event",
+							  G_CALLBACK (gtk_imhtml_tip_paint), imhtml);
+
+	gtk_widget_ensure_style (imhtml->tip_window);
+	layout = gtk_widget_create_pango_layout(imhtml->tip_window, imhtml->tip);
+	font = pango_font_get_metrics(pango_context_load_font(pango_layout_get_context(layout),
+														  imhtml->tip_window->style->font_desc),
+														  NULL);
+
+
+	pango_layout_get_pixel_size(layout, &scr_w, NULL);
+	gap = PANGO_PIXELS((pango_font_metrics_get_ascent(font) + 
+					   pango_font_metrics_get_descent(font))/ 4);
+
+	if (gap < 2)
+		gap = 2;
+	baseline_skip = PANGO_PIXELS(pango_font_metrics_get_ascent(font) + 
+								pango_font_metrics_get_descent(font));
+	w = 8 + scr_w;
+	h = 8 + baseline_skip;
+
+	gdk_window_get_pointer (NULL, &x, &y, NULL);
+	if (GTK_WIDGET_NO_WINDOW (GTK_WIDGET(imhtml)))
+		y += GTK_WIDGET(imhtml)->allocation.y;
+
+	scr_w = gdk_screen_width();
+
+	x -= ((w >> 1) + 4);
+
+	if ((x + w) > scr_w)
+		x -= (x + w) - scr_w;
+	else if (x < 0)
+		x = 0;
+
+	y = y + PANGO_PIXELS(pango_font_metrics_get_ascent(font) + 
+						pango_font_metrics_get_descent(font));
+
+	gtk_widget_set_size_request (imhtml->tip_window, w, h);
+	gtk_widget_show (imhtml->tip_window);
+	gtk_window_move (GTK_WINDOW(imhtml->tip_window), x, y);
+
+	pango_font_metrics_unref(font);
+	g_object_unref(layout);
+
+	return FALSE;
+}
+
+gboolean gtk_motion_event_notify(GtkWidget *imhtml, GdkEventMotion *event, gpointer data)
+{	
+	GtkTextIter iter;
+	GdkWindow *win = event->window;
+	int x, y;
+	char *tip = NULL;
+	GSList *tags = NULL, *templist = NULL;
+	gdk_window_get_pointer(GTK_WIDGET(imhtml)->window, NULL, NULL, NULL);
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(imhtml), GTK_TEXT_WINDOW_WIDGET,
+										  event->x, event->y, &x, &y);
+	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(imhtml), &iter, x, y);
+	tags = gtk_text_iter_get_tags(&iter);
+
+	templist = tags;
+	while (templist) {
+		GtkTextTag *tag = templist->data;
+		tip = g_object_get_data(G_OBJECT(tag), "link_url");
+		if (tip)
+			break;
+		templist = templist->next;
+	}
+		
+	if (GTK_IMHTML(imhtml)->tip) {
+		if ((tip == GTK_IMHTML(imhtml)->tip)) {
+			return FALSE;
+		}
+		/* We've left the cell.  Remove the timeout and create a new one below */
+		if (GTK_IMHTML(imhtml)->tip_window) {
+			gtk_widget_destroy(GTK_IMHTML(imhtml)->tip_window);
+			GTK_IMHTML(imhtml)->tip_window = NULL;
+		}
+		gdk_window_set_cursor(win, GTK_IMHTML(imhtml)->arrow_cursor);
+		if (GTK_IMHTML(imhtml)->tip_timer)
+			g_source_remove(GTK_IMHTML(imhtml)->tip_timer);
+		GTK_IMHTML(imhtml)->tip_timer = 0;
+	}
+	
+	if(tip){
+		gdk_window_set_cursor(win, GTK_IMHTML(imhtml)->hand_cursor);
+		GTK_IMHTML(imhtml)->tip_timer = g_timeout_add (TOOLTIP_TIMEOUT, 
+							       gtk_imhtml_tip, imhtml);
+	}
+	
+	GTK_IMHTML(imhtml)->tip = tip;
+	g_slist_free(tags);
+	return FALSE;
+}
+
+gboolean gtk_leave_event_notify(GtkWidget *imhtml, GdkEventCrossing *event, gpointer data)
+{
+	/* when leaving the widget, clear any current & pending tooltips and restore the cursor */
+	if (GTK_IMHTML(imhtml)->tip_window) {
+		gtk_widget_destroy(GTK_IMHTML(imhtml)->tip_window);
+		GTK_IMHTML(imhtml)->tip_window = NULL;
+	}
+	if (GTK_IMHTML(imhtml)->tip_timer) {
+		g_source_remove(GTK_IMHTML(imhtml)->tip_timer);
+		GTK_IMHTML(imhtml)->tip_timer = 0;
+	}
+	gdk_window_set_cursor(event->window, GTK_IMHTML(imhtml)->arrow_cursor);
+
+	/* propogate the event normally */
+	return FALSE;
+}
+
+/*
+ * XXX - This should be removed eventually.
+ *
+ * This function exists to work around a gross bug in GtkTextView.  
+ * Basically, we short circuit ctrl+a and ctrl+end because they make 
+ * el program go boom.
+ *
+ * It's supposed to be fixed in gtk2.2.  You can view the bug report at 
+ * http://bugzilla.gnome.org/show_bug.cgi?id=107939
+ */
+gboolean gtk_key_pressed_cb(GtkWidget *imhtml, GdkEventKey *event, gpointer data)
+{
+	if (event->state & GDK_CONTROL_MASK)
+		switch (event->keyval) {
+			case 'a':
+				return TRUE;
+				break;
+
+			case GDK_Home:
+				return TRUE;
+				break;
+
+			case GDK_End:
+				return TRUE;
+				break;
+		}
+
+	return FALSE;
+}
+
+
 
 static GtkTextViewClass *parent_class = NULL;
 
-
 /* GtkIMHtml has one signal--URL_CLICKED */
 enum {
 	URL_CLICKED,
@@ -242,6 +424,7 @@ static void gtk_imhtml_init (GtkIMHtml *imhtml)
 	g_signal_connect(G_OBJECT(imhtml), "size-allocate", G_CALLBACK(gtk_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(imhtml), "motion-notify-event", G_CALLBACK(gtk_motion_event_notify), NULL);
 	g_signal_connect(G_OBJECT(imhtml), "leave-notify-event", G_CALLBACK(gtk_leave_event_notify), NULL);
+	g_signal_connect(G_OBJECT(imhtml), "key_press_event", G_CALLBACK(gtk_key_pressed_cb), NULL);
 	gtk_widget_add_events(GTK_WIDGET(imhtml), GDK_LEAVE_NOTIFY_MASK);
 
 	imhtml->tip = NULL;
@@ -294,11 +477,15 @@ static void url_open(GtkWidget *w, struct url_data *data) {
 	g_free(data->url);
 	g_free(data);
 }
+
 static void url_copy(GtkWidget *w, gchar *url) {
 	GtkClipboard *clipboard;
 
 	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
 	gtk_clipboard_set_text(clipboard, url, -1);
+
+	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text(clipboard, url, -1);
 }
 
 /* The callback for an event on a link tag. */
@@ -366,71 +553,6 @@ gboolean tag_event(GtkTextTag *tag, GObject *imhtml, GdkEvent *event, GtkTextIte
 		return FALSE; /* Let clicks go through if we didn't catch anything */
 }
 
-gboolean gtk_motion_event_notify(GtkWidget *imhtml, GdkEventMotion *event, gpointer data)
-{	
-	GtkTextIter iter;
-	GdkWindow *win = event->window;
-	int x, y;
-	char *tip = NULL;
-	GSList *tags = NULL, *templist = NULL;
-	gdk_window_get_pointer(GTK_WIDGET(imhtml)->window, NULL, NULL, NULL);
-	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(imhtml), GTK_TEXT_WINDOW_WIDGET,
-										  event->x, event->y, &x, &y);
-	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(imhtml), &iter, x, y);
-	tags = gtk_text_iter_get_tags(&iter);
-
-	templist = tags;
-	while (templist) {
-		GtkTextTag *tag = templist->data;
-		tip = g_object_get_data(G_OBJECT(tag), "link_url");
-		if (tip)
-			break;
-		templist = templist->next;
-	}
-		
-	if (GTK_IMHTML(imhtml)->tip) {
-		if ((tip == GTK_IMHTML(imhtml)->tip)) {
-			return FALSE;
-		}
-		/* We've left the cell.  Remove the timeout and create a new one below */
-		if (GTK_IMHTML(imhtml)->tip_window) {
-			gtk_widget_destroy(GTK_IMHTML(imhtml)->tip_window);
-			GTK_IMHTML(imhtml)->tip_window = NULL;
-		}
-		gdk_window_set_cursor(win, GTK_IMHTML(imhtml)->arrow_cursor);
-		if (GTK_IMHTML(imhtml)->tip_timer)
-			g_source_remove(GTK_IMHTML(imhtml)->tip_timer);
-		GTK_IMHTML(imhtml)->tip_timer = 0;
-	}
-	
-	if(tip){
-		gdk_window_set_cursor(win, GTK_IMHTML(imhtml)->hand_cursor);
-		GTK_IMHTML(imhtml)->tip_timer = g_timeout_add (TOOLTIP_TIMEOUT, 
-							       gtk_imhtml_tip, imhtml);
-	}
-	
-	GTK_IMHTML(imhtml)->tip = tip;
-	g_slist_free(tags);
-	return FALSE;
-}
-
-gboolean gtk_leave_event_notify(GtkWidget *imhtml, GdkEventCrossing *event, gpointer data)
-{
-	/* when leaving the widget, clear any current & pending tooltips and restore the cursor */
-	if (GTK_IMHTML(imhtml)->tip_window) {
-		gtk_widget_destroy(GTK_IMHTML(imhtml)->tip_window);
-		GTK_IMHTML(imhtml)->tip_window = NULL;
-	}
-	if (GTK_IMHTML(imhtml)->tip_timer) {
-		g_source_remove(GTK_IMHTML(imhtml)->tip_timer);
-		GTK_IMHTML(imhtml)->tip_timer = 0;
-	}
-	gdk_window_set_cursor(event->window, GTK_IMHTML(imhtml)->arrow_cursor);
-
-	/* propogate the event normally */
-	return FALSE;
-}
-
 /* this isn't used yet
 static void
 gtk_smiley_tree_remove (GtkSmileyTree     *tree,
@@ -478,7 +600,7 @@ gtk_smiley_tree_lookup (GtkSmileyTree *tree,
 
 		pos = strchr (t->values->str, *x);
 		if (pos)
-			t = t->children [(int) pos - (int) t->values->str];
+			t = t->children [(int) pos - GPOINTER_TO_INT(t->values->str)];
 		else
 			break;
 
@@ -518,7 +640,7 @@ gtk_imhtml_is_smiley (GtkIMHtml   *imhtml,
 		      gint        *len)
 {
 	GtkSmileyTree *tree;
-	FontDetail *font;
+	GtkIMHtmlFontDetail *font;
 	char *sml = NULL;
 
 	if (fonts) {
@@ -563,7 +685,7 @@ gtk_smiley_tree_image (GtkIMHtml     *imhtml,
 		
 		pos = strchr (t->values->str, *x);
 		if (pos) {
-			t = t->children [(int) pos - (int) t->values->str];
+			t = t->children [(int) pos - GPOINTER_TO_INT(t->values->str)];
 		} else {
 			return sml ? gtk_smiley_tree_image(imhtml, NULL, text) : NULL;
 		}
@@ -833,7 +955,7 @@ gtk_imhtml_get_html_opt (gchar       *tag,
                                 gtk_text_buffer_apply_tag(imhtml->text_buffer, texttag, &siter, &iter); \
                         } \
                         if (fonts) { \
-                                 FontDetail *fd = fonts->data; \
+                                 GtkIMHtmlFontDetail *fd = fonts->data; \
                                  if (fd->fore) { \
 	                                    texttag = gtk_text_buffer_create_tag(imhtml->text_buffer, NULL, "foreground", fd->fore, NULL); \
                                             gtk_text_buffer_apply_tag(imhtml->text_buffer, texttag, &siter, &iter); \
@@ -843,7 +965,7 @@ gtk_imhtml_get_html_opt (gchar       *tag,
                                             gtk_text_buffer_apply_tag(imhtml->text_buffer, texttag, &siter, &iter); \
                                  } \
                                  if (fd->face) { \
-                                            texttag = gtk_text_buffer_create_tag(imhtml->text_buffer, NULL, "font", fd->face, NULL); \
+                                            texttag = gtk_text_buffer_create_tag(imhtml->text_buffer, NULL, "family", fd->face, NULL); \
                                             gtk_text_buffer_apply_tag(imhtml->text_buffer, texttag, &siter, &iter); \
                                  } \
                                  if (fd->size) { \
@@ -1036,13 +1158,13 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 				case 26:        /* HR */
 				case 42:        /* HR (opt) */
 					ws[wpos++] = '\n';
-					scalable = gaim_hr_new();
+					scalable = gtk_imhtml_hr_new();
 					NEW_BIT(NEW_SCALABLE_BIT);
 					ws[wpos++] = '\n';
 					break;
 				case 27:	/* /FONT */
 					if (fonts) {
-						FontDetail *font = fonts->data;
+						GtkIMHtmlFontDetail *font = fonts->data;
 						NEW_BIT (NEW_TEXT_BIT);
 						fonts = g_slist_remove (fonts, font);
 						if (font->face)
@@ -1088,7 +1210,7 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 				case 43:	/* FONT (opt) */
 					{
 						gchar *color, *back, *face, *size, *sml;
-						FontDetail *font, *oldfont = NULL;
+						GtkIMHtmlFontDetail *font, *oldfont = NULL;
 						color = gtk_imhtml_get_html_opt (tag, "COLOR=");
 						back = gtk_imhtml_get_html_opt (tag, "BACK=");
 						face = gtk_imhtml_get_html_opt (tag, "FACE=");
@@ -1099,7 +1221,7 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 						
 						NEW_BIT (NEW_TEXT_BIT);
 						
-						font = g_new0 (FontDetail, 1);
+						font = g_new0 (GtkIMHtmlFontDetail, 1);
 						if (fonts)
 							oldfont = fonts->data;
 
@@ -1137,6 +1259,8 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 							} else if (isdigit (*size)) {
 								sscanf (size, "%hd", &font->size);
 							}
+							if (font->size > 100)
+								font->size = 100;
 						} else if (oldfont)
 							font->size = oldfont->size;
 						g_free(size);
@@ -1194,7 +1318,7 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 							} else {
 								imagepb = gdk_pixbuf_loader_get_pixbuf(load);
 								if (imagepb) {
-									scalable = gaim_im_image_new(imagepb, g_strdup(src));
+									scalable = gtk_imhtml_image_new(imagepb, g_strdup(src));
 									NEW_BIT(NEW_SCALABLE_BIT);
 									g_object_unref(imagepb);
 								}
@@ -1248,7 +1372,7 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 			c++;
 			pos++;
 		} else if (imhtml->show_smileys && (gtk_imhtml_is_smiley (imhtml, fonts, c, &smilelen) || gtk_imhtml_is_smiley(imhtml, NULL, c, &smilelen))) {
-			FontDetail *fd;
+			GtkIMHtmlFontDetail *fd;
 			gchar *sml = NULL;
 			if (fonts) {
 				fd = fonts->data;
@@ -1277,7 +1401,7 @@ GString* gtk_imhtml_append_text (GtkIMHtml        *imhtml,
 	}
 	
 	while (fonts) {
-		FontDetail *font = fonts->data;
+		GtkIMHtmlFontDetail *font = fonts->data;
 		fonts = g_slist_remove (fonts, font);
 		if (font->face)
 			g_free (font->face);
@@ -1388,128 +1512,15 @@ void gtk_imhtml_page_down (GtkIMHtml *imhtml)
 	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(imhtml), &iter, 0, TRUE, 0, 0);
 }
 
-static gint
-gtk_imhtml_tip_paint (GtkIMHtml *imhtml)
+/* GtkIMHtmlScalable, gtk_imhtml_image, gtk_imhtml_hr */
+GtkIMHtmlScalable *gtk_imhtml_image_new(GdkPixbuf *img, gchar *filename)
 {
-	PangoLayout *layout;
-
-	g_return_val_if_fail(GTK_IS_IMHTML(imhtml), FALSE);
-
-	layout = gtk_widget_create_pango_layout(imhtml->tip_window, imhtml->tip);
-
-	gtk_paint_flat_box (imhtml->tip_window->style, imhtml->tip_window->window, 
-						GTK_STATE_NORMAL, GTK_SHADOW_OUT, NULL, imhtml->tip_window,
-						"tooltip", 0, 0, -1, -1);
-
-	gtk_paint_layout (imhtml->tip_window->style, imhtml->tip_window->window, GTK_STATE_NORMAL,
-					  FALSE, NULL, imhtml->tip_window, NULL, 4, 4, layout);
-
-	g_object_unref(layout);
-	return FALSE;
-}
-
-static gint
-gtk_imhtml_tip (gpointer data)
-{
-	GtkIMHtml *imhtml = data;
-	PangoFontMetrics *font;
-	PangoLayout *layout;
-
-	gint gap, x, y, h, w, scr_w, baseline_skip;
-
-	g_return_val_if_fail(GTK_IS_IMHTML(imhtml), FALSE);
-
-	if (!imhtml->tip || !GTK_WIDGET_DRAWABLE (GTK_WIDGET(imhtml))) {
-		imhtml->tip_timer = 0;
-		return FALSE;
-	}
-	
-	if (imhtml->tip_window){
-		gtk_widget_destroy (imhtml->tip_window);
-		imhtml->tip_window = NULL;
-	}
-
-	imhtml->tip_timer = 0;
-	imhtml->tip_window = gtk_window_new (GTK_WINDOW_POPUP);
-	gtk_widget_set_app_paintable (imhtml->tip_window, TRUE);
-	gtk_window_set_resizable (GTK_WINDOW (imhtml->tip_window), FALSE);
-	gtk_widget_set_name (imhtml->tip_window, "gtk-tooltips");
-	g_signal_connect_swapped (G_OBJECT (imhtml->tip_window), "expose_event",
-							  G_CALLBACK (gtk_imhtml_tip_paint), imhtml);
-
-	gtk_widget_ensure_style (imhtml->tip_window);
-	layout = gtk_widget_create_pango_layout(imhtml->tip_window, imhtml->tip);
-	font = pango_font_get_metrics(pango_context_load_font(pango_layout_get_context(layout),
-														  imhtml->tip_window->style->font_desc),
-														  NULL);
-
-
-	pango_layout_get_pixel_size(layout, &scr_w, NULL);
-	gap = PANGO_PIXELS((pango_font_metrics_get_ascent(font) + 
-					   pango_font_metrics_get_descent(font))/ 4);
-
-	if (gap < 2)
-		gap = 2;
-	baseline_skip = PANGO_PIXELS(pango_font_metrics_get_ascent(font) + 
-								pango_font_metrics_get_descent(font));
-	w = 8 + scr_w;
-	h = 8 + baseline_skip;
-
-	gdk_window_get_pointer (NULL, &x, &y, NULL);
-	if (GTK_WIDGET_NO_WINDOW (GTK_WIDGET(imhtml)))
-		y += GTK_WIDGET(imhtml)->allocation.y;
-
-	scr_w = gdk_screen_width();
-
-	x -= ((w >> 1) + 4);
-
-	if ((x + w) > scr_w)
-		x -= (x + w) - scr_w;
-	else if (x < 0)
-		x = 0;
-
-	y = y + PANGO_PIXELS(pango_font_metrics_get_ascent(font) + 
-						pango_font_metrics_get_descent(font));
-
-	gtk_widget_set_size_request (imhtml->tip_window, w, h);
-	gtk_widget_show (imhtml->tip_window);
-	gtk_window_move (GTK_WINDOW(imhtml->tip_window), x, y);
-
-	pango_font_metrics_unref(font);
-	g_object_unref(layout);
-
-	return FALSE;
-}
-
-static gboolean gtk_size_allocate_cb(GtkIMHtml *widget, GtkAllocation *alloc, gpointer user_data)
-{
-	GdkRectangle rect;
-
-	gtk_text_view_get_visible_rect(GTK_TEXT_VIEW(widget), &rect);
-	if(widget->old_rect.width != rect.width || widget->old_rect.height != rect.height){
-		GList *iter = GTK_IMHTML(widget)->scalables;
-
-		while(iter){
-			GtkIMHtmlScalable *scale = GTK_IMHTML_SCALABLE(iter->data);
-			scale->scale(scale, rect.width, rect.height);
-
-			iter = iter->next;
-		}
-	}
-
-	widget->old_rect = rect;
-	return FALSE;
-}
-
-/* GtkIMHtmlScalable, gaim_im_image, gaim_hr */
-GtkIMHtmlScalable *gaim_im_image_new(GdkPixbuf *img, gchar *filename)
-{
-	gaim_im_image *im_image = g_malloc(sizeof(gaim_im_image));
+	GtkIMHtmlImage *im_image = g_malloc(sizeof(GtkIMHtmlImage));
 	GtkImage *image = GTK_IMAGE(gtk_image_new_from_pixbuf(img));
 
-	GTK_IMHTML_SCALABLE(im_image)->scale = gaim_im_image_scale;
-	GTK_IMHTML_SCALABLE(im_image)->add_to = gaim_im_image_add_to;
-	GTK_IMHTML_SCALABLE(im_image)->free = gaim_im_image_free;
+	GTK_IMHTML_SCALABLE(im_image)->scale = gtk_imhtml_image_scale;
+	GTK_IMHTML_SCALABLE(im_image)->add_to = gtk_imhtml_image_add_to;
+	GTK_IMHTML_SCALABLE(im_image)->free = gtk_imhtml_image_free;
 
 	im_image->pixbuf = img;
 	im_image->image = image;
@@ -1522,9 +1533,9 @@ GtkIMHtmlScalable *gaim_im_image_new(GdkPixbuf *img, gchar *filename)
 	return GTK_IMHTML_SCALABLE(im_image);
 }
 
-void gaim_im_image_scale(GtkIMHtmlScalable *scale, int width, int height)
+void gtk_imhtml_image_scale(GtkIMHtmlScalable *scale, int width, int height)
 {
-	gaim_im_image *image = (gaim_im_image *)scale;
+	GtkIMHtmlImage *image = (GtkIMHtmlImage *)scale;
 
 	if(image->width > width || image->height > height){
 		GdkPixbuf *new_image = NULL;
@@ -1548,73 +1559,33 @@ void gaim_im_image_scale(GtkIMHtmlScalable *scale, int width, int height)
 	}
 }
 
-void gaim_im_image_free(GtkIMHtmlScalable *scale)
-{
-	gaim_im_image *image = (gaim_im_image *)scale;
-
-	g_object_unref(image->pixbuf);
-	g_free(image->filename);
-	g_free(scale);
-}
-
-void gaim_im_image_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *imhtml, GtkTextIter *iter)
-{
-	gaim_im_image *image = (gaim_im_image *)scale;
-	GtkWidget *box = gtk_event_box_new();
-	GtkTextChildAnchor *anchor = gtk_text_buffer_create_child_anchor(imhtml->text_buffer, iter);
-
-	gtk_container_add(GTK_CONTAINER(box), GTK_WIDGET(image->image));
-
-	gtk_widget_show(GTK_WIDGET(image->image));
-	gtk_widget_show(box);
-
-	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(imhtml), box, anchor);
-	g_signal_connect(G_OBJECT(box), "event", G_CALLBACK(gaim_im_image_clicked), image);
-}
-
-GtkIMHtmlScalable *gaim_hr_new()
-{
-	gaim_hr *hr = g_malloc(sizeof(gaim_hr));
-
-	GTK_IMHTML_SCALABLE(hr)->scale = gaim_hr_scale;
-	GTK_IMHTML_SCALABLE(hr)->add_to = gaim_hr_add_to;
-	GTK_IMHTML_SCALABLE(hr)->free = gaim_hr_free;
-
-	hr->sep = gtk_hseparator_new();
-	gtk_widget_set_size_request(hr->sep, 5000, 2);
-	gtk_widget_show(hr->sep);
-
-	return GTK_IMHTML_SCALABLE(hr);
-}
-
-void gaim_hr_scale(GtkIMHtmlScalable *scale, int width, int height)
-{
-	gtk_widget_set_size_request(((gaim_hr *)scale)->sep, width, 2);
-}
-
-void gaim_hr_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *imhtml, GtkTextIter *iter)
-{
-	gaim_hr *hr = (gaim_hr *)scale;
-	GtkTextChildAnchor *anchor = gtk_text_buffer_create_child_anchor(imhtml->text_buffer, iter);
-
-	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(imhtml), hr->sep, anchor);
-}
-
-void gaim_hr_free(GtkIMHtmlScalable *scale)
-{
-/*	gtk_widget_destroy(((gaim_hr *)scale)->sep); */
-	g_free(scale);
-}
-
 static void write_img_to_file(GtkWidget *w, GtkFileSelection *sel)
 {
 	const gchar *filename = gtk_file_selection_get_filename(sel);
-	gaim_im_image *image = g_object_get_data(G_OBJECT(sel), "gaim_im_image");
+	gchar *dirname;
+	GtkIMHtmlImage *image = g_object_get_data(G_OBJECT(sel), "GtkIMHtmlImage");
 	gchar *type = NULL;
 	GError *error = NULL;
 #if GTK_CHECK_VERSION(2,2,0)
 	GSList *formats = gdk_pixbuf_get_formats();
+#else
+	char *basename = g_path_get_basename(filename);
+	char *ext = strrchr(basename, '.');
+#endif
 
+	if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
+		/* append a / if needed */
+		if (filename[strlen(filename) - 1] != '/') {
+			dirname = g_strconcat(filename, "/", NULL);
+		} else {
+			dirname = g_strdup(filename);
+		}
+		gtk_file_selection_set_filename(sel, dirname);
+		g_free(dirname);
+		return;
+	}
+
+#if GTK_CHECK_VERSION(2,2,0)
 	while(formats){
 		GdkPixbufFormat *format = formats->data;
 		gchar **extensions = gdk_pixbuf_format_get_extensions(format);
@@ -1643,9 +1614,6 @@ static void write_img_to_file(GtkWidget *w, GtkFileSelection *sel)
 	g_slist_free(formats);
 #else
 	/* this is really ugly code, but I think it will work */
-	char *basename = g_path_get_basename(filename);
-	char *ext = strrchr(basename, '.');
-
 	if(ext) {
 		ext++;
 		if(!g_ascii_strcasecmp(ext, "jpeg") || !g_ascii_strcasecmp(ext, "jpg"))
@@ -1661,7 +1629,7 @@ static void write_img_to_file(GtkWidget *w, GtkFileSelection *sel)
 	   it's a png */
 	if(!type){
 		gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						_("Gaim was unable to guess the image type base on the file extension supplied.  Defaulting to PNG."));
+						_("Unable to guess the image type based on the file extension supplied.  Defaulting to PNG."));
 		type = g_strdup("png");
 	}
 
@@ -1676,12 +1644,12 @@ static void write_img_to_file(GtkWidget *w, GtkFileSelection *sel)
 	g_free(type);
 }
 
-static void gaim_im_image_save(GtkWidget *w, gaim_im_image *image)
+static void gtk_imhtml_image_save(GtkWidget *w, GtkIMHtmlImage *image)
 {
-	GtkWidget *sel = gtk_file_selection_new(_("Gaim - Save Image"));
+	GtkWidget *sel = gtk_file_selection_new(_("Save Image"));
 
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(sel), image->filename);
-	g_object_set_data(G_OBJECT(sel), "gaim_im_image", image);
+	g_object_set_data(G_OBJECT(sel), "GtkIMHtmlImage", image);
 	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(sel)->ok_button), "clicked",
 					 G_CALLBACK(write_img_to_file), sel);
 
@@ -1693,7 +1661,7 @@ static void gaim_im_image_save(GtkWidget *w, gaim_im_image *image)
 	gtk_widget_show(sel);
 }
 
-static gboolean gaim_im_image_clicked(GtkWidget *w, GdkEvent *event, gaim_im_image *image)
+static gboolean gtk_imhtml_image_clicked(GtkWidget *w, GdkEvent *event, GtkIMHtmlImage *image)
 {
 	GdkEventButton *event_button = (GdkEventButton *) event;
 
@@ -1707,7 +1675,7 @@ static gboolean gaim_im_image_clicked(GtkWidget *w, GdkEvent *event, gaim_im_ima
 			img = gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU);
 			item = gtk_image_menu_item_new_with_mnemonic(text);
 			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), img);
-			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(gaim_im_image_save), image);
+			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(gtk_imhtml_image_save), image);
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
 			gtk_widget_show_all(menu);
@@ -1725,3 +1693,60 @@ static gboolean gaim_im_image_clicked(GtkWidget *w, GdkEvent *event, gaim_im_ima
 		return FALSE; /* Let clicks go through if we didn't catch anything */
 
 }
+void gtk_imhtml_image_free(GtkIMHtmlScalable *scale)
+{
+	GtkIMHtmlImage *image = (GtkIMHtmlImage *)scale;
+
+	g_object_unref(image->pixbuf);
+	g_free(image->filename);
+	g_free(scale);
+}
+
+void gtk_imhtml_image_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *imhtml, GtkTextIter *iter)
+{
+	GtkIMHtmlImage *image = (GtkIMHtmlImage *)scale;
+	GtkWidget *box = gtk_event_box_new();
+	GtkTextChildAnchor *anchor = gtk_text_buffer_create_child_anchor(imhtml->text_buffer, iter);
+
+	gtk_container_add(GTK_CONTAINER(box), GTK_WIDGET(image->image));
+
+	gtk_widget_show(GTK_WIDGET(image->image));
+	gtk_widget_show(box);
+
+	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(imhtml), box, anchor);
+	g_signal_connect(G_OBJECT(box), "event", G_CALLBACK(gtk_imhtml_image_clicked), image);
+}
+
+GtkIMHtmlScalable *gtk_imhtml_hr_new()
+{
+	GtkIMHtmlHr *hr = g_malloc(sizeof(GtkIMHtmlHr));
+
+	GTK_IMHTML_SCALABLE(hr)->scale = gtk_imhtml_hr_scale;
+	GTK_IMHTML_SCALABLE(hr)->add_to = gtk_imhtml_hr_add_to;
+	GTK_IMHTML_SCALABLE(hr)->free = gtk_imhtml_hr_free;
+
+	hr->sep = gtk_hseparator_new();
+	gtk_widget_set_size_request(hr->sep, 5000, 2);
+	gtk_widget_show(hr->sep);
+
+	return GTK_IMHTML_SCALABLE(hr);
+}
+
+void gtk_imhtml_hr_scale(GtkIMHtmlScalable *scale, int width, int height)
+{
+	gtk_widget_set_size_request(((GtkIMHtmlHr *)scale)->sep, width, 2);
+}
+
+void gtk_imhtml_hr_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *imhtml, GtkTextIter *iter)
+{
+	GtkIMHtmlHr *hr = (GtkIMHtmlHr *)scale;
+	GtkTextChildAnchor *anchor = gtk_text_buffer_create_child_anchor(imhtml->text_buffer, iter);
+
+	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(imhtml), hr->sep, anchor);
+}
+
+void gtk_imhtml_hr_free(GtkIMHtmlScalable *scale)
+{
+/*	gtk_widget_destroy(((GtkIMHtmlHr *)scale)->sep); */
+	g_free(scale);
+}
