@@ -19,61 +19,54 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-#ifdef GAIM_PLUGINS
-#ifndef _WIN32
-#include <dlfcn.h>
-#endif
-#endif /* GAIM_PLUGINS */
-#include <gtk/gtk.h>
-#ifndef _WIN32
-#include <gdk/gdkx.h>
-#include <unistd.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/un.h>
-#include <sys/wait.h>
-#endif /* !_WIN32 */
-#include <gdk/gdk.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <ctype.h>
+#include "gtkinternal.h"
+
+#include "account.h"
+#include "conversation.h"
+#include "core.h"
+#include "debug.h"
+#include "ft.h"
+#include "log.h"
+#include "notify.h"
+#include "prefs.h"
 #include "prpl.h"
+#include "pounce.h"
 #include "sound.h"
-#include "gaim.h"
-#include "gaim-socket.h"
-#include "gtklist.h"
+#include "status.h"
+#include "util.h"
+
+#include "gtkaccount.h"
+#include "gtkblist.h"
+#include "gtkconn.h"
+#include "gtkconv.h"
+#include "gtkdebug.h"
+#include "gtkft.h"
+#include "gtknotify.h"
+#include "gtkpounce.h"
+#include "gtkprefs.h"
+#include "gtkprivacy.h"
+#include "gtkrequest.h"
+#include "gtksound.h"
+#include "gtkutils.h"
+#include "stock.h"
+
+#include "ui.h"
+
 #if HAVE_SIGNAL_H
-#include <signal.h>
+# include <signal.h>
 #endif
+
 #include "locale.h"
 #include <getopt.h>
 
-#ifdef _WIN32
-#include "win32dep.h"
-#endif
+extern void load_prefs();
+extern void load_pounces();
 
 static GtkWidget *name;
 static GtkWidget *pass;
 
-GList *log_conversations = NULL;
-GList *buddy_pounces = NULL;
-GSList *away_messages = NULL;
-GSList *message_queue = NULL;
-GSList *unread_message_queue = NULL;
-GSList *away_time_queue = NULL;
-
 GtkWidget *mainwindow = NULL;
 
-
 int opt_away = 0;
 int docklet_count = 0;
 char *opt_away_arg = NULL;
@@ -104,43 +97,10 @@ static int ignore_sig_list[] = {
 };
 #endif
 
-void do_quit()
-{
-	/* captain's log, stardate... */
-	system_log(log_quit, NULL, NULL, OPT_LOG_BUDDY_SIGNON | OPT_LOG_MY_SIGNON);
-
-	/* the self destruct sequence has been initiated */
-	plugin_event(event_quit);
-
-	/* transmission ends */
-	signoff_all();
-
-	/* record what we have before we blow it away... */
-	save_prefs();
-
-#ifdef GAIM_PLUGINS
-	/* jettison cargo */
-	remove_all_plugins();
-#endif
-
-#ifdef USE_PERL
-	/* yup, perl too */
-	perl_end();
-#endif
-
-#ifdef USE_SM
-	/* unplug */
-	session_end();
-#endif
-
-	/* and end it all... */
-	gtk_main_quit();
-}
-
 static guint snd_tmout = 0;
 static gboolean sound_timeout(gpointer data)
 {
-	gaim_sound_set_login_mute(FALSE);
+	gaim_gtk_sound_set_login_mute(FALSE);
 	snd_tmout = 0;
 	return FALSE;
 }
@@ -148,13 +108,14 @@ static gboolean sound_timeout(gpointer data)
 /* we need to do this for Oscar because serv_login only starts the login
  * process, it doesn't end there. gaim_setup will be called later from
  * oscar.c, after the buddy list is made and serv_finish_login is called */
-void gaim_setup(struct gaim_connection *gc)
+void gaim_setup(GaimConnection *gc)
 {
-	if ((sound_options & OPT_SOUND_LOGIN) && (sound_options & OPT_SOUND_SILENT_SIGNON)) {
+	if (gaim_prefs_get_bool("/gaim/gtk/sound/enabled/login")
+			&& gaim_prefs_get_bool("/gaim/gtk/sound/silent_signon")) {
 		if(snd_tmout) {
 			g_source_remove(snd_tmout);
 		}
-		gaim_sound_set_login_mute(TRUE);
+		gaim_gtk_sound_set_login_mute(TRUE);
 		snd_tmout = g_timeout_add(10000, sound_timeout, NULL);
 	}
 }
@@ -164,18 +125,19 @@ static gboolean domiddleclick(GtkWidget *w, GdkEventButton *event, gpointer null
 	if (event->button != 2)
 		return FALSE;
 
-	auto_login();
+	gaim_accounts_auto_login(GAIM_GTK_UI);
+
 	return TRUE;
 }
 
 static void dologin(GtkWidget *widget, GtkWidget *w)
 {
-	struct gaim_account *account;
+	GaimAccount *account;
 	const char *username = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(name)->entry));
 	const char *password = gtk_entry_get_text(GTK_ENTRY(pass));
 
-	if (!strlen(username)) {
-		do_error_dialog(_("Please enter your login."), NULL, GAIM_ERROR);
+	if (*username == '\0') {
+		gaim_notify_error(NULL, NULL, _("Please enter your login."), NULL);
 		return;
 	}
 
@@ -183,12 +145,16 @@ static void dologin(GtkWidget *widget, GtkWidget *w)
 	 * them, they just have to use the account editor to sign in 
 	 * the second one */
 
-	account = gaim_account_find(username, -1);
-	if (!account)
-		account = gaim_account_new(username, DEFAULT_PROTO, OPT_ACCT_REM_PASS);
-	g_snprintf(account->password, sizeof account->password, "%s", password);
-	save_prefs();
-	serv_login(account);
+	account = gaim_accounts_find(username, -1);
+	if (!account) {
+		account = gaim_account_new(username, "prpl-oscar");
+		gaim_account_set_remember_password(account, TRUE);
+		gaim_accounts_add(account);
+	}
+
+	gaim_account_set_password(account, (*password != '\0') ? password : NULL);
+
+	gaim_account_connect(account);
 }
 
 /* <name> is a comma-separated list of names, or NULL
@@ -200,28 +166,24 @@ static void dologin(GtkWidget *widget, GtkWidget *w)
 */
 static int dologin_named(char *name)
 {
-	struct gaim_account *account;
+	GaimAccount *account;
 	char **names, **n;
 	int retval = -1;
 
 	if (name !=NULL) {	/* list of names given */
 		names = g_strsplit(name, ",", 32);
 		for (n = names; *n != NULL; n++) {
-			account = gaim_account_find(*n, -1);
+			account = gaim_accounts_find(*n, -1);
 			if (account) {	/* found a user */
-				if (account->options & OPT_ACCT_REM_PASS) {
-					retval = 0;
-					serv_login(account);
-				}
+				retval = 0;
+				gaim_account_connect(account);
 			}
 		}
 		g_strfreev(names);
 	} else {		/* no name given, use default */
-		account = (struct gaim_account *)gaim_accounts->data;
-		if (account->options & OPT_ACCT_REM_PASS) {
-			retval = 0;
-			serv_login(account);
-		}
+		account = (GaimAccount *)gaim_accounts_get_all()->data;
+		retval = 0;
+		gaim_account_connect(account);
 	}
 
 	return retval;
@@ -230,24 +192,20 @@ static int dologin_named(char *name)
 
 static void doenter(GtkWidget *widget, GtkWidget *w)
 {
-	if (widget == name) {
-		gtk_entry_set_text(GTK_ENTRY(pass), "");
-		gtk_editable_select_region(GTK_EDITABLE(GTK_COMBO(name)->entry), 0, 0);
-		gtk_widget_grab_focus(pass);
-	} else if (widget == pass) {
-		dologin(widget, w);
-	}
+	gtk_entry_set_text(GTK_ENTRY(pass), "");
+	gtk_editable_select_region(GTK_EDITABLE(GTK_COMBO(name)->entry), 0, 0);
+	gtk_widget_grab_focus(pass);
 }
 
 
 static void combo_changed(GtkWidget *w, GtkWidget *combo)
 {
 	const char *txt = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));
-	struct gaim_account *account;
+	GaimAccount *account;
 
-	account = gaim_account_find(txt, -1);
+	account = gaim_accounts_find(txt, -1);
 
-	if (account && account->options & OPT_ACCT_REM_PASS) {
+	if (account && gaim_account_get_remember_password(account)) {
 		gtk_entry_set_text(GTK_ENTRY(pass), account->password);
 	} else {
 		gtk_entry_set_text(GTK_ENTRY(pass), "");
@@ -257,15 +215,15 @@ static void combo_changed(GtkWidget *w, GtkWidget *combo)
 
 static GList *combo_user_names()
 {
-	GSList *accts = gaim_accounts;
+	GList *accts = gaim_accounts_get_all();
 	GList *tmp = NULL;
-	struct gaim_account *account;
+	GaimAccount *account;
 
 	if (!accts)
 		return g_list_append(NULL, _("<New User>"));
 
 	while (accts) {
-		account = (struct gaim_account *)accts->data;
+		account = (GaimAccount *)accts->data;
 		tmp = g_list_append(tmp, account->username);
 		accts = accts->next;
 	}
@@ -281,7 +239,7 @@ static void login_window_closed(GtkWidget *w, GdkEvent *ev, gpointer d)
 #endif
 		gtk_widget_hide(mainwindow);
 	} else
-		do_quit();
+		gaim_core_quit();
 }
 
 void show_login()
@@ -293,7 +251,6 @@ void show_login()
 	GtkWidget *label;
 	GtkWidget *vbox2;
 	GList *tmp;
-	char *filename;
 
 	/* Do we already have a main window opened? If so, bring it back, baby... ribs... yeah */
 	if (mainwindow) {
@@ -315,9 +272,7 @@ void show_login()
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(mainwindow), vbox);
 
-	filename = g_build_filename(DATADIR, "pixmaps", "gaim", "logo.png", NULL);
-	image = gtk_image_new_from_file(filename);
-	g_free(filename);
+	image = gtk_image_new_from_stock(GAIM_STOCK_LOGO, gtk_icon_size_from_name(GAIM_ICON_SIZE_LOGO));
 	gtk_box_pack_start(GTK_BOX(vbox), image, FALSE, FALSE, 0);
 
 	vbox2 = gtk_vbox_new(FALSE, 0);
@@ -348,7 +303,7 @@ void show_login()
 	pass = gtk_entry_new();
 	gtk_entry_set_visibility(GTK_ENTRY(pass), FALSE);
 	g_signal_connect(G_OBJECT(pass), "activate",
-					 G_CALLBACK(doenter), mainwindow);
+					 G_CALLBACK(dologin), mainwindow);
 	gtk_box_pack_start(GTK_BOX(vbox2), pass, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, TRUE, 0);
 
@@ -357,23 +312,23 @@ void show_login()
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
 
 	/* And now for the buttons */
-	button = gaim_pixbuf_button(_("Accounts"), "accounts.png", GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("Accounts"), GAIM_STOCK_ACCOUNTS, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(account_editor), mainwindow);
+					 G_CALLBACK(gaim_gtk_accounts_window_show), mainwindow);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
 #ifdef NO_MULTI
 	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
 #endif
 
-	button = gaim_pixbuf_button(_("Settings"), "preferences.png", GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("Preferences"), GTK_STOCK_PREFERENCES, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(show_prefs), mainwindow);
+					 G_CALLBACK(gaim_gtk_prefs_show), mainwindow);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
-	button = gaim_pixbuf_button(_("Sign On"), "signon.png", GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("Sign on"), GAIM_STOCK_SIGN_ON, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(dologin), mainwindow);
@@ -381,9 +336,10 @@ void show_login()
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
 	/* Now grab the focus that we need */
-	if (gaim_accounts) {
-		struct gaim_account *account = gaim_accounts->data;
-		if (account->options & OPT_ACCT_REM_PASS) {
+	if (gaim_accounts_get_all()) {
+		GaimAccount *account = gaim_accounts_get_all()->data;
+
+		if (gaim_account_get_remember_password(account)) {
 			combo_changed(NULL, name);
 			gtk_widget_grab_focus(button);
 		} else {
@@ -403,32 +359,28 @@ void sighandler(int sig)
 {
 	switch (sig) {
 	case SIGHUP:
-		debug_printf("caught signal %d\n", sig);
-		signoff_all(NULL, NULL);
+		gaim_debug(GAIM_DEBUG_WARNING, "sighandler",
+				   "Caught signal %d\n", sig);
+		gaim_connections_disconnect_all();
 		break;
 	case SIGSEGV:
-		core_quit();
 #ifndef DEBUG
 		fprintf(stderr, "Gaim has segfaulted and attempted to dump a core file.\n"
 			"This is a bug in the software and has happened through\n"
 			"no fault of your own.\n\n"
 			"It is possible that this bug is already fixed in CVS.\n"
-			"You can get a tarball of CVS from the Gaim website, at\n"
-			WEBSITE "gaim-CVS.tar.gz.\n\n"
-			"If you are already using CVS, or can reproduce the crash\n"
-			"using the CVS version, please notify the gaim maintainers\n"
-			"by reporting a bug at\n"
-			WEBSITE "bug.php\n\n"
+			"If you can reproduce the crash, please notify the gaim\n" 
+			"maintainers by reporting a bug at\n"
+			GAIM_WEBSITE "bug.php\n\n"
 			"Please make sure to specify what you were doing at the time,\n"
 			"and post the backtrace from the core file. If you do not know\n"
 			"how to get the backtrace, please get instructions at\n"
-			WEBSITE "gdb.php. If you need further\n"
+			GAIM_WEBSITE "gdb.php. If you need further\n"
 			"assistance, please IM either RobFlynn or SeanEgn and\n"
 			"they can help you.\n");
 #else
 		fprintf(stderr, "Oh no!  Segmentation fault!\n");
 		/*g_on_error_query (g_get_prgname());*/
-		exit(1);
 #endif
 		abort();
 		break;
@@ -439,88 +391,22 @@ void sighandler(int sig)
 #endif
 		break;
 	default:
-		debug_printf("caught signal %d\n", sig);
-		signoff_all(NULL, NULL);
-#ifdef GAIM_PLUGINS
-		remove_all_plugins();
-#endif
+		gaim_debug(GAIM_DEBUG_WARNING, "sighandler",
+				   "Caught signal %d\n", sig);
+		gaim_connections_disconnect_all();
+
+		gaim_plugins_unload_all();
+
 		if (gtk_main_level())
 			gtk_main_quit();
-		core_quit();
 		exit(0);
 	}
 }
 #endif
 
-#ifndef _WIN32
-static gboolean socket_readable(GIOChannel *source, GIOCondition cond, gpointer ud)
-{
-	guchar type;
-	guchar subtype;
-	guint32 len;
-	guchar *data;
-	guint32 x;
-	GError *error;
-
-	debug_printf("Core says: ");
-	g_io_channel_read_chars(source, &type, sizeof(type), &x, &error);
-	if(error)
-		g_error_free(error);
-	if (x == 0) {
-		debug_printf("CORE IS GONE!\n");
-		g_io_channel_shutdown(source, TRUE, &error);
-		if(error)
-			g_free(error);
-		return FALSE;
-	}
-	debug_printf("%d ", type);
-	g_io_channel_read_chars(source, &subtype, sizeof(subtype), &x, &error);
-	if(error)
-		g_error_free(error);
-	if (x == 0) {
-		debug_printf("CORE IS GONE!\n");
-		g_io_channel_shutdown(source, TRUE, &error);
-		if(error)
-			g_error_free(error);
-		return FALSE;
-	}
-	debug_printf("%d ", subtype);
-	g_io_channel_read_chars(source, (guchar *)&len, sizeof(len), &x, &error);
-	if(error)
-		g_error_free(error);
-	if (x == 0) {
-		debug_printf("CORE IS GONE!\n");
-		g_io_channel_shutdown(source, TRUE, &error);
-		if(error)
-			g_error_free(error);
-		return FALSE;
-	}
-	debug_printf("(%d bytes)\n", len);
-
-	data = g_malloc(len);
-	g_io_channel_read_chars(source, data, len, &x, &error);
-	if(error)
-		g_error_free(error);
-	if (x != len) {
-		debug_printf("CORE IS GONE! (read %d/%d bytes)\n", x, len);
-		g_free(data);
-		g_io_channel_shutdown(source, TRUE, &error);
-		if(error)
-			g_error_free(error);
-		return FALSE;
-	}
-
-	g_free(data);
-	return TRUE;
-}
-#endif /* _WIN32 */
-
 static int ui_main()
 {
 #ifndef _WIN32
-	GIOChannel *channel;
-	int UI_fd;
-	char name[256];
 	GList *icons = NULL;
 	GdkPixbuf *icon = NULL;
 	char *icon_path;
@@ -534,11 +420,11 @@ static int ui_main()
 		}
 	}
 
-	setup_stock();
+	gaim_gtk_blist_setup_sort_methods();
 
 #ifndef _WIN32
 	/* use the nice PNG icon for all the windows */
-	icon_path = g_build_filename(DATADIR, "pixmaps", "gaim.png", NULL);
+	icon_path = g_build_filename(DATADIR, "pixmaps", "gaim", "icons", "online.png", NULL);
 	icon = gdk_pixbuf_new_from_file(icon_path, NULL);
 	g_free(icon_path);
 	if (icon) {
@@ -547,16 +433,9 @@ static int ui_main()
 		g_object_unref(G_OBJECT(icon));
 		g_list_free(icons);
 	} else {
-		debug_printf("Failed to load default window icon!\n");
+		gaim_debug(GAIM_DEBUG_ERROR, "ui_main",
+				   "Failed to load the default window icon!\n");
 	}
-
-	g_snprintf(name, sizeof(name), "%s" G_DIR_SEPARATOR_S "gaim_%s.%d", g_get_tmp_dir(), g_get_user_name(), gaim_session);
-	UI_fd = gaim_connect_to_session(0);
-	if (UI_fd < 0)
-		return 1;
-
-	channel = g_io_channel_unix_new(UI_fd);
-	g_io_add_watch(channel, G_IO_IN | G_IO_HUP | G_IO_ERR, socket_readable, NULL);
 #endif
 
 	return 0;
@@ -564,45 +443,101 @@ static int ui_main()
 
 static void set_first_user(char *name)
 {
-	struct gaim_account *account;
+	GaimAccount *account;
 
-	account = gaim_account_find(name, -1);
+	account = gaim_accounts_find(name, -1);
 
-	if (!account) {		/* new user */
-		account = g_new0(struct gaim_account, 1);
-		g_snprintf(account->username, sizeof(account->username), "%s", name);
-		account->protocol = DEFAULT_PROTO;
-		gaim_accounts = g_slist_prepend(gaim_accounts, account);
-	} else {		/* user already exists */
-		gaim_accounts = g_slist_remove(gaim_accounts, account);
-		gaim_accounts = g_slist_prepend(gaim_accounts, account);
+	if (account == NULL) { /* new user */
+		account = gaim_account_new(name, "prpl-oscar");
+		gaim_accounts_add(account);
 	}
-	save_prefs();
+
+	/* Place it as the first user. */
+	gaim_accounts_reorder(account, 0);
 }
 
 #ifdef _WIN32
 /* WIN32 print and log handlers */
 
 static void gaim_dummy_print( const gchar* string ) {
-        return;
+	return;
 }
 
 static void gaim_dummy_log_handler (const gchar    *domain,
 				    GLogLevelFlags  flags,
 				    const gchar    *msg,
 				    gpointer        user_data) {
-        return;
+	return;
 }
 
 static void gaim_log_handler (const gchar    *domain,
 			      GLogLevelFlags  flags,
 			      const gchar    *msg,
 			      gpointer        user_data) {
-        debug_printf("%s - %s\n", domain, msg);
+	gaim_debug(GAIM_DEBUG_MISC, "log", "%s - %s\n", domain, msg);
 	g_log_default_handler(domain, flags, msg, user_data);
 }
 #endif /* _WIN32 */
 
+static void
+debug_init(void)
+{
+	gaim_set_debug_ui_ops(gaim_get_gtk_debug_ui_ops());
+	gaim_gtk_debug_init();
+}
+
+static void
+gaim_gtk_ui_init(void)
+{
+	/* Set the UI operation structures. */
+	gaim_set_win_ui_ops(gaim_get_gtk_window_ui_ops());
+	gaim_set_xfer_ui_ops(gaim_get_gtk_xfer_ui_ops());
+	gaim_set_blist_ui_ops(gaim_get_gtk_blist_ui_ops());
+	gaim_set_notify_ui_ops(gaim_get_gtk_notify_ui_ops());
+	gaim_set_privacy_ui_ops(gaim_gtk_privacy_get_ui_ops());
+	gaim_set_request_ui_ops(gaim_get_gtk_request_ui_ops());
+	gaim_set_sound_ui_ops(gaim_get_gtk_sound_ui_ops());
+	gaim_set_connection_ui_ops(gaim_get_gtk_connection_ui_ops());
+
+	gaim_gtk_prefs_init();
+	gaim_gtk_blist_init();
+	gaim_gtk_conversation_init();
+	gaim_gtk_pounces_init();
+	gaim_gtk_privacy_init();
+	gaim_gtk_xfers_init();
+}
+
+static void
+gaim_gtk_quit(void)
+{
+	/* XXX? */
+
+	/* captain's log, stardate... */
+	system_log(log_quit, NULL, NULL, OPT_LOG_BUDDY_SIGNON | OPT_LOG_MY_SIGNON);
+
+#ifdef USE_SM
+	/* unplug */
+	session_end();
+#endif
+
+	/* and end it all... */
+	gtk_main_quit();
+}
+
+static GaimCoreUiOps core_ops =
+{
+	gaim_gtk_prefs_init,
+	debug_init,
+	gaim_gtk_ui_init,
+	gaim_gtk_quit
+};
+
+static GaimCoreUiOps *
+gaim_get_gtk_core_ui_ops(void)
+{
+	return &core_ops;
+}
+
 /* FUCKING GET ME A TOWEL! */
 #ifdef _WIN32
 int gaim_main(int argc, char *argv[])
@@ -613,6 +548,7 @@ int main(int argc, char *argv[])
 	int opt_acct = 0, opt_help = 0, opt_version = 0, opt_login = 0, opt_nologin = 0, dologin_ret = -1;
 	char *opt_user_arg = NULL, *opt_login_arg = NULL;
 	char *opt_session_arg = NULL;
+	char *plugin_search_paths[3];
 #if HAVE_SIGNAL_H
 	int sig_indx;	/* for setting up signal catching */
 	sigset_t sigset;
@@ -638,7 +574,6 @@ int main(int argc, char *argv[])
 #ifdef DEBUG
 	opt_debug = 1;
 #endif
-
 #ifdef ENABLE_NLS
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -653,19 +588,19 @@ int main(int argc, char *argv[])
 	 * declare a handler for. thanks JSeymour and Vann.            */
 	if (sigemptyset(&sigset)) {
 		char errmsg[BUFSIZ];
-		sprintf(errmsg, "Warning: couldn't initialise empty signal set");
+		snprintf(errmsg, BUFSIZ, "Warning: couldn't initialise empty signal set");
 		perror(errmsg);
 	}
 	for(sig_indx = 0; catch_sig_list[sig_indx] != -1; ++sig_indx) {
 		if((prev_sig_disp = signal(catch_sig_list[sig_indx], sighandler)) == SIG_ERR) {
 			char errmsg[BUFSIZ];
-			sprintf(errmsg, "Warning: couldn't set signal %d for catching",
+			snprintf(errmsg, BUFSIZ, "Warning: couldn't set signal %d for catching",
 				catch_sig_list[sig_indx]);
 			perror(errmsg);
 		}
 		if(sigaddset(&sigset, catch_sig_list[sig_indx])) {
 			char errmsg[BUFSIZ];
-			sprintf(errmsg, "Warning: couldn't include signal %d for unblocking",
+			snprintf(errmsg, BUFSIZ, "Warning: couldn't include signal %d for unblocking",
 				catch_sig_list[sig_indx]);
 			perror(errmsg);
 		}
@@ -673,7 +608,7 @@ int main(int argc, char *argv[])
 	for(sig_indx = 0; ignore_sig_list[sig_indx] != -1; ++sig_indx) {
 		if((prev_sig_disp = signal(ignore_sig_list[sig_indx], SIG_IGN)) == SIG_ERR) {
 			char errmsg[BUFSIZ];
-			sprintf(errmsg, "Warning: couldn't set signal %d to ignore",
+			snprintf(errmsg, BUFSIZ, "Warning: couldn't set signal %d to ignore",
 				ignore_sig_list[sig_indx]);
 			perror(errmsg);
 		}
@@ -681,7 +616,7 @@ int main(int argc, char *argv[])
 
 	if (sigprocmask(SIG_UNBLOCK, &sigset, NULL)) {
 		char errmsg[BUFSIZ];
-		sprintf(errmsg, "Warning: couldn't unblock signals");
+		snprintf(errmsg, BUFSIZ, "Warning: couldn't unblock signals");
 		perror(errmsg);
 	}		
 #endif
@@ -858,23 +793,55 @@ int main(int argc, char *argv[])
 		printf("Gaim %s\n",VERSION);
 		return 0;
 	}
-	
-#if GAIM_PLUGINS || USE_PERL
-	gaim_probe_plugins();
-#endif
-		
+
+	gaim_gtk_stock_init();
+
+	gaim_set_core_ui_ops(gaim_get_gtk_core_ui_ops());
+
+	if (!gaim_core_init(GAIM_GTK_UI)) {
+		fprintf(stderr,
+				"Initialization of the Gaim core failed. Dumping core.\n"
+				"Please report this!\n");
+		abort();
+	}
+
+	plugin_search_paths[0] = LIBDIR;
+	plugin_search_paths[1] = gaim_user_dir();
+	plugin_search_paths[2] = g_strdup_printf("%s/plugins", gaim_user_dir());
+
+	gaim_plugins_set_search_paths(sizeof(plugin_search_paths) /
+								  sizeof(*plugin_search_paths),
+								  plugin_search_paths);
+
+	g_free(plugin_search_paths[2]);
+
+	gaim_plugins_probe(NULL);
+
+	gaim_prefs_load();
+
+
+	/* we only read ~/.gaimrc (load_prefs()) if there is no accounts.xml
+	 * since prefs.xml existed alongside ~/.gaim in 0.64 */
+	if (!gaim_accounts_load()) {
+		load_prefs();
+		gaim_prefs_sync();
+	}
+
+	/* load plugins we had when we quit */
+	gaim_plugins_load_saved("/gaim/gtk/plugins/loaded");
+
+	gaim_pounces_load();
+	gaim_status_load();
+
 #ifdef _WIN32
 	/* Various win32 initializations */
 	wgaim_init();
 #endif
 
-	/* Set the UI operation structures. */
-	gaim_set_win_ui_ops(gaim_get_gtk_window_ui_ops());
-	gaim_set_xfer_ui_ops(gaim_get_gtk_xfer_ui_ops());
-	gaim_set_blist_ui_ops(gaim_get_gtk_blist_ui_ops());
+	gaim_set_blist(gaim_blist_new());
+	gaim_blist_load();
 
-	load_prefs();
-	core_main();
+	load_pounces();
 	ui_main();
 
 #ifdef USE_SM
@@ -892,10 +859,8 @@ int main(int argc, char *argv[])
 		opt_user_arg = NULL;
 	}
 
-	if (misc_options & OPT_MISC_DEBUG)
-		show_debug();
-
-	static_proto_init();
+	if (gaim_prefs_get_bool("/gaim/gtk/debug/enabled"))
+		gaim_gtk_debug_window_show();
 
 	/* deal with --login */
 	if (opt_login) {
@@ -906,17 +871,16 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (!opt_acct && !opt_nologin && gaim_session == 0)
-		auto_login();
+	if (!opt_acct && !opt_nologin)
+		gaim_accounts_auto_login(GAIM_GTK_UI);
 
 	if (opt_acct) {
-		account_editor(NULL, NULL);
-	} else if ((dologin_ret == -1) && !connections)
+		gaim_gtk_accounts_window_show();
+	} else if ((dologin_ret == -1) && !gaim_connections_get_all())
 		show_login();
 
 	gtk_main();
-	core_quit();
-	gaim_sound_quit();
+	gaim_sound_shutdown();
 #ifdef _WIN32
 	wgaim_cleanup();
 #endif
