@@ -1,6 +1,6 @@
 /*
  * gaim - Gadu-Gadu Protocol Plugin
- * $Id: gg.c,v 1.58 2003/05/18 19:59:00 chipx86 Exp $
+ * $Id: gg.c,v 1.74 2003/08/25 15:42:38 lschiere Exp $
  *
  * Copyright (C) 2001 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
  * 
@@ -19,42 +19,25 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
+#include "internal.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifndef _WIN32
-#include <netdb.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/socket.h>
-#else
-#include <winsock.h>
-#endif
-
-#include <errno.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <time.h>
-#include <sys/stat.h>
-#include <ctype.h>
 /* Library from EKG (Eksperymentalny Klient Gadu-Gadu) */
 #include "libgg.h"
-#include "gaim.h"
+
+#include "account.h"
+#include "accountopt.h"
+#include "debug.h"
 #include "multi.h"
-#include "core.h"
-#include "prpl.h"
+#include "notify.h"
 #include "proxy.h"
+#include "prpl.h"
+#include "server.h"
+#include "util.h"
 
-#ifdef _WIN32
-#include "win32dep.h"
-#endif
+/* XXX for g_show_info_text(), WEBSITE, and stuff */
+#include "gaim.h"
 
-#define USEROPT_NICK 0
+#define GG_CONNECT_STEPS 5
 
 #define AGG_BUF_LEN 1024
 
@@ -83,16 +66,13 @@
 
 #define UC_NORMAL 2
 
-/* for win32 compatability */
-G_MODULE_IMPORT GSList *connections;
-
 struct agg_data {
 	struct gg_session *sess;
 	int own_status;
 };
 
 struct agg_http {
-	struct gaim_connection *gc;
+	GaimConnection *gc;
 	gchar *request;
 	gchar *form;
 	gchar *host;
@@ -119,9 +99,11 @@ static gint args_compare(gconstpointer a, gconstpointer b)
 	return g_ascii_strcasecmp((const gchar *)a,(const gchar *)b);
 }
 
-static gboolean allowed_uin(struct gaim_connection *gc, char *uin)
+static gboolean allowed_uin(GaimConnection *gc, char *uin)
 {
-	switch (gc->account->permdeny) {
+	GaimAccount *account = gaim_connection_get_account(gc);
+
+	switch (account->perm_deny) {
 	case 1:
 		/* permit all, deny none */
 		return TRUE;
@@ -148,7 +130,7 @@ static gboolean allowed_uin(struct gaim_connection *gc, char *uin)
 	}
 }
 
-static char *handle_errcode(struct gaim_connection *gc, int errcode)
+static char *handle_errcode(GaimConnection *gc, int errcode)
 {
 	static char msg[AGG_BUF_LEN];
 
@@ -176,12 +158,12 @@ static char *handle_errcode(struct gaim_connection *gc, int errcode)
 		break;
 	}
 
-	hide_login_progress(gc, msg);
+	gaim_connection_error(gc, msg);
 
 	return msg;
 }
 
-static void agg_set_away(struct gaim_connection *gc, char *state, char *msg)
+static void agg_set_away(GaimConnection *gc, const char *state, const char *msg)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	int status = gd->own_status;
@@ -250,7 +232,7 @@ static gchar *get_away_text(int uc)
 	}
 }
 
-static GList *agg_away_states(struct gaim_connection *gc)
+static GList *agg_away_states(GaimConnection *gc)
 {
 	GList *m = NULL;
 
@@ -265,7 +247,7 @@ static GList *agg_away_states(struct gaim_connection *gc)
 }
 
 /* Enhance these functions, more options and such stuff */
-static GList *agg_buddy_menu(struct gaim_connection *gc, const char *who)
+static GList *agg_buddy_menu(GaimConnection *gc, const char *who)
 {
 	GList *m = NULL;
 	struct proto_buddy_menu *pbm;
@@ -288,25 +270,24 @@ static GList *agg_buddy_menu(struct gaim_connection *gc, const char *who)
 
 static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 {
-	struct gaim_connection *gc = data;
+	GaimConnection *gc = data;
 	struct agg_data *gd = gc->proto_data;
 	struct gg_event *e;
 
-	debug_printf("main_callback enter: begin\n");
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "main_callback enter: begin\n");
 
 	if (gd->sess->fd != source)
 		gd->sess->fd = source;
 
 	if (source == 0) {
-		hide_login_progress(gc, _("Could not connect"));
-		signoff(gc);
+		gaim_connection_error(gc, _("Could not connect"));
 		return;
 	}
 
 	if (!(e = gg_watch_fd(gd->sess))) {
-		debug_printf("main_callback: gg_watch_fd failed - CRITICAL!\n");
-		hide_login_progress(gc, _("Unable to read socket"));
-		signoff(gc);
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "main_callback: gg_watch_fd failed - CRITICAL!\n");
+		gaim_connection_error(gc, _("Unable to read socket"));
 		return;
 	}
 
@@ -315,13 +296,13 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 		/* do nothing */
 		break;
 	case GG_EVENT_CONN_SUCCESS:
-		debug_printf("main_callback: CONNECTED AGAIN!?\n");
+		gaim_debug(GAIM_DEBUG_WARNING, "gg",
+				   "main_callback: CONNECTED AGAIN!?\n");
 		break;
 	case GG_EVENT_CONN_FAILED:
 		if (gc->inpa)
 			gaim_input_remove(gc->inpa);
 		handle_errcode(gc, e->event.failure);
-		signoff(gc);
 		break;
 	case GG_EVENT_MSG:
 		{
@@ -391,11 +372,13 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 		}
 		break;
 	case GG_EVENT_ACK:
-		debug_printf("main_callback: message %d to %lu sent with status %d\n",
+		gaim_debug(GAIM_DEBUG_MISC, "gg",
+				   "main_callback: message %d to %lu sent with status %d\n",
 			     e->event.ack.seq, e->event.ack.recipient, e->event.ack.status);
 		break;
 	default:
-		debug_printf("main_callback: unsupported event %d\n", e->type);
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "main_callback: unsupported event %d\n", e->type);
 		break;
 	}
 	gg_free_event(e);
@@ -403,59 +386,60 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 
 void login_callback(gpointer data, gint source, GaimInputCondition cond)
 {
-	struct gaim_connection *gc = data;
+	GaimConnection *gc = data;
 	struct agg_data *gd = gc->proto_data;
 	struct gg_event *e;
 
-	debug_printf("GG login_callback...\n");
-	if (!g_slist_find(connections, gc)) {
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "login_callback...\n");
+	if (!g_list_find(gaim_connections_get_all(), gc)) {
 		close(source);
 		return;
 	}
-	debug_printf("Found GG connection\n");
-	
+
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "Found GG connection\n");
+
 	if (source == 0) {
-		hide_login_progress(gc, _("Unable to connect."));
-		signoff(gc);
+		gaim_connection_error(gc, _("Unable to connect."));
 		return;
 	}
 
 	gd->sess->fd = source;
 
-	debug_printf("Source is valid.\n");
+	gaim_debug(GAIM_DEBUG_MISC, "gg", "Source is valid.\n");
 	if (gc->inpa == 0) {
-		debug_printf("login_callback.. checking gc->inpa .. is 0.. setting fd watch\n");
+		gaim_debug(GAIM_DEBUG_MISC, "gg",
+				   "login_callback.. checking gc->inpa .. is 0.. setting fd watch\n");
 		gc->inpa = gaim_input_add(gd->sess->fd, GAIM_INPUT_READ, login_callback, gc);
-		debug_printf("Adding watch on fd\n"); 
+		gaim_debug(GAIM_DEBUG_INFO, "gg", "Adding watch on fd\n"); 
 	}
-	debug_printf("Checking State.\n");
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "Checking State.\n");
 	switch (gd->sess->state) {
 	case GG_STATE_READING_DATA:
-		set_login_progress(gc, 2, _("Reading data"));
+		gaim_connection_update_progress(gc, _("Reading data"), 2, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_CONNECTING_GG:
-		set_login_progress(gc, 3, _("Balancer handshake"));
+		gaim_connection_update_progress(gc, _("Balancer handshake"), 3, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_READING_KEY:
-		set_login_progress(gc, 4, _("Reading server key"));
+		gaim_connection_update_progress(gc, _("Reading server key"), 4, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_READING_REPLY:
-		set_login_progress(gc, 5, _("Exchanging key hash"));
+		gaim_connection_update_progress(gc, _("Exchanging key hash"), 5, GG_CONNECT_STEPS);
 		break;
 	default:
-		debug_printf("No State found\n");
+		gaim_debug(GAIM_DEBUG_INFO, "gg", "No State found\n");
 		break;
 	}
-	debug_printf("gg_watch_fd\n");
+	gaim_debug(GAIM_DEBUG_MISC, "gg", "gg_watch_fd\n");
 	if (!(e = gg_watch_fd(gd->sess))) {
-		debug_printf("login_callback: gg_watch_fd failed - CRITICAL!\n");
-		hide_login_progress(gc, _("Critical error in GG library\n"));
-		signoff(gc);
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "login_callback: gg_watch_fd failed - CRITICAL!\n");
+		gaim_connection_error(gc, _("Critical error in GG library\n"));
 		return;
 	}
 
 	/* If we are GG_STATE_CONNECTING_GG then we still need to connect, as
-	   we could not use proxy_connect in libgg.c */
+	   we could not use gaim_proxy_connect in libgg.c */
 	switch( gd->sess->state ) {
 	case GG_STATE_CONNECTING_GG:
 		{
@@ -467,10 +451,9 @@ void login_callback(gpointer data, gint source, GaimInputCondition cond)
 
 			ip.s_addr = gd->sess->server_ip;
 
-			if (proxy_connect(gc->account, inet_ntoa(ip), gd->sess->port, login_callback, gc) < 0) {
+			if (gaim_proxy_connect(gc->account, inet_ntoa(ip), gd->sess->port, login_callback, gc) < 0) {
 				g_snprintf(buf, sizeof(buf), _("Connect to %s failed"), inet_ntoa(ip));
-				hide_login_progress(gc, buf);
-				signoff(gc);
+				gaim_connection_error(gc, buf);
 				return;
 			}
 			break;
@@ -479,11 +462,12 @@ void login_callback(gpointer data, gint source, GaimInputCondition cond)
 		/* Set new watch on login server ip */
 		if(gc->inpa)
 			gc->inpa = gaim_input_add(gd->sess->fd, GAIM_INPUT_READ, login_callback, gc);
-		debug_printf("Setting watch on connection with login server.\n"); 
+		gaim_debug(GAIM_DEBUG_INFO, "gg",
+				   "Setting watch on connection with login server.\n"); 
 		break;
 	}/* end switch() */
 
-	debug_printf("checking gg_event\n");
+	gaim_debug(GAIM_DEBUG_MISC, "gg", "checking gg_event\n");
 	switch (e->type) {
 	case GG_EVENT_NONE:
 		/* nothing */
@@ -495,7 +479,7 @@ void login_callback(gpointer data, gint source, GaimInputCondition cond)
 		gc->inpa = gaim_input_add(gd->sess->fd, GAIM_INPUT_READ, main_callback, gc);
 
 		/* Our signon is complete */
-		account_online(gc);
+		gaim_connection_set_state(gc, GAIM_CONNECTED);
 		serv_finish_login(gc);
 
 		break;
@@ -503,45 +487,40 @@ void login_callback(gpointer data, gint source, GaimInputCondition cond)
 		gaim_input_remove(gc->inpa);
 		gc->inpa = 0;
 		handle_errcode(gc, e->event.failure);
-		signoff(gc);
 		break;
 	default:
-		debug_printf("no gg_event\n");
+		gaim_debug(GAIM_DEBUG_MISC, "gg", "no gg_event\n");
 		break;
 	}
-	debug_printf("Returning from login_callback\n");
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "Returning from login_callback\n");
 	gg_free_event(e);
 }
 
-static void agg_keepalive(struct gaim_connection *gc)
+static void agg_keepalive(GaimConnection *gc)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	if (gg_ping(gd->sess) < 0) {
-		hide_login_progress(gc, _("Unable to ping server"));
-		signoff(gc);
+		gaim_connection_error(gc, _("Unable to ping server"));
 		return;
 	}
 }
 
-static void agg_login(struct gaim_account *account)
+static void agg_login(GaimAccount *account)
 {
-	struct gaim_connection *gc = new_gaim_conn(account);
+	GaimConnection *gc = gaim_account_get_connection(account);
 	struct agg_data *gd = gc->proto_data = g_new0(struct agg_data, 1);
 	char buf[80];
 
+#if 0
 	gc->checkbox = _("Send as message");
+#endif
 
 	gd->sess = g_new0(struct gg_session, 1);
 
-	if (account->proto_opt[USEROPT_NICK][0])
-		g_snprintf(gc->displayname, sizeof(gc->displayname), "%s",
-			   account->proto_opt[USEROPT_NICK]);
-
-	set_login_progress(gc, 1, _("Looking up GG server"));
+	gaim_connection_update_progress(gc, _("Looking up GG server"), 1, GG_CONNECT_STEPS);
 
 	if (invalid_uin(account->username)) {
-		hide_login_progress(gc, _("Invalid Gadu-Gadu UIN specified"));
-		signoff(gc);
+		gaim_connection_error(gc, _("Invalid Gadu-Gadu UIN specified"));
 		return;
 	}
 
@@ -549,13 +528,13 @@ static void agg_login(struct gaim_account *account)
 
 	/*
 	   if (gg_login(gd->sess, strtol(user->username, (char **)NULL, 10), user->password, 1) < 0) {
-	   debug_printf("uin=%u, pass=%s", strtol(user->username, (char **)NULL, 10), user->password); 
-	   hide_login_progress(gc, "Unable to connect.");
+	   gaim_debug(GAIM_DEBUG_MISC, "gg", "uin=%u, pass=%s", strtol(user->username, (char **)NULL, 10), user->password); 
+	   gaim_connection_error(gc, _("Unable to connect."));
 	   signoff(gc);
 	   return;
 	   }
 
-	   gg_login() sucks for me, so I'm using proxy_connect()
+	   gg_login() sucks for me, so I'm using gaim_proxy_connect()
 	 */
 
 	gd->sess->uin = (uin_t) strtol(account->username, (char **)NULL, 10);
@@ -563,15 +542,14 @@ static void agg_login(struct gaim_account *account)
 	gd->sess->state = GG_STATE_CONNECTING;
 	gd->sess->check = GG_CHECK_WRITE;
 	gd->sess->async = 1;
-	if (proxy_connect(account, GG_APPMSG_HOST, GG_APPMSG_PORT, login_callback, gc) < 0) {
+	if (gaim_proxy_connect(account, GG_APPMSG_HOST, GG_APPMSG_PORT, login_callback, gc) < 0) {
 		g_snprintf(buf, sizeof(buf), _("Connect to %s failed"), GG_APPMSG_HOST);
-		hide_login_progress(gc, buf);
-		signoff(gc);
+		gaim_connection_error(gc, buf);
 		return;
 	}
 }
 
-static void agg_close(struct gaim_connection *gc)
+static void agg_close(GaimConnection *gc)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	if (gc->inpa)
@@ -582,7 +560,7 @@ static void agg_close(struct gaim_connection *gc)
 	g_free(gc->proto_data);
 }
 
-static int agg_send_im(struct gaim_connection *gc, const char *who, const char *msg, int len, int flags)
+static int agg_send_im(GaimConnection *gc, const char *who, const char *msg, int len, GaimImFlags flags)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	gchar *imsg;
@@ -596,8 +574,7 @@ static int agg_send_im(struct gaim_connection *gc, const char *who, const char *
 
 	if (strlen(msg) > 0) {
 		imsg = charset_convert(msg, "UTF-8", "CP1250");
-		if (gg_send_message(gd->sess, (flags & IM_FLAG_CHECKBOX)
-				    ? GG_CLASS_MSG : GG_CLASS_CHAT,
+		if (gg_send_message(gd->sess, GG_CLASS_CHAT,
 				    strtol(who, (char **)NULL, 10), imsg) < 0)
 			return -1;
 		g_free(imsg);
@@ -605,7 +582,7 @@ static int agg_send_im(struct gaim_connection *gc, const char *who, const char *
 	return 1;
 }
 
-static void agg_add_buddy(struct gaim_connection *gc, const char *who)
+static void agg_add_buddy(GaimConnection *gc, const char *who)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	if (invalid_uin(who))
@@ -613,7 +590,7 @@ static void agg_add_buddy(struct gaim_connection *gc, const char *who)
 	gg_add_notify(gd->sess, strtol(who, (char **)NULL, 10));
 }
 
-static void agg_rem_buddy(struct gaim_connection *gc, char *who, char *group)
+static void agg_rem_buddy(GaimConnection *gc, const char *who, const char *group)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	if (invalid_uin(who))
@@ -621,7 +598,7 @@ static void agg_rem_buddy(struct gaim_connection *gc, char *who, char *group)
 	gg_remove_notify(gd->sess, strtol(who, (char **)NULL, 10));
 }
 
-static void agg_add_buddies(struct gaim_connection *gc, GList *whos)
+static void agg_add_buddies(GaimConnection *gc, GList *whos)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	uin_t *userlist = NULL;
@@ -643,7 +620,7 @@ static void agg_add_buddies(struct gaim_connection *gc, GList *whos)
 	}
 }
 
-static void search_results(struct gaim_connection *gc, gchar *webdata)
+static void search_results(GaimConnection *gc, gchar *webdata)
 {
 	gchar **webdata_tbl;
 	gchar *buf;
@@ -651,7 +628,7 @@ static void search_results(struct gaim_connection *gc, gchar *webdata)
 	int i, j;
 
 	if ((ptr = strstr(webdata, "query_results:")) == NULL || (ptr = strchr(ptr, '\n')) == NULL) {
-		debug_printf("search_callback: pubdir result [%s]\n", webdata);
+		gaim_debug(GAIM_DEBUG_MISC, "gg", "search_callback: pubdir result [%s]\n", webdata);
 		gaim_notify_error(gc, NULL, _("Couldn't get search results"), NULL);
 		return;
 	}
@@ -696,7 +673,7 @@ static void search_results(struct gaim_connection *gc, gchar *webdata)
 			break;
 		case 3:
 			ibuf =
-			    g_strconcat(oldibuf, "<B>", _("Second Name"), ":</B> ", p, "<BR>\n", NULL);
+			    g_strconcat(oldibuf, "<B>", _("Last Name"), ":</B> ", p, "<BR>\n", NULL);
 			g_free(oldibuf);
 			break;
 		case 4:
@@ -750,7 +727,7 @@ static void search_results(struct gaim_connection *gc, gchar *webdata)
 	g_free(buf);
 }
 
-static void import_buddies_server_results(struct gaim_connection *gc, gchar *webdata)
+static void import_buddies_server_results(GaimConnection *gc, gchar *webdata)
 {
 	gchar *ptr;
 	gchar **users_tbl;
@@ -763,7 +740,7 @@ static void import_buddies_server_results(struct gaim_connection *gc, gchar *web
 	}
 
 	if ((ptr = strstr(webdata, "get_results:")) == NULL || (ptr = strchr(ptr, ':')) == NULL) {
-		debug_printf("import_buddies_server_results: import buddies result [%s]\n", webdata);
+		gaim_debug(GAIM_DEBUG_MISC, "gg", "import_buddies_server_results: import buddies result [%s]\n", webdata);
 		gaim_notify_error(gc, NULL,
 						  _("Couldn't Import Buddy List from Server"), NULL);
 		return;
@@ -778,7 +755,8 @@ static void import_buddies_server_results(struct gaim_connection *gc, gchar *web
 		gchar *name, *show;
 
 		if (strlen(users_tbl[i])==0) {
-			debug_printf("import_buddies_server_results: users_tbl[i] is empty\n");
+			gaim_debug(GAIM_DEBUG_MISC, "gg",
+					   "import_buddies_server_results: users_tbl[i] is empty\n");
 			continue;
 		}
 
@@ -792,7 +770,8 @@ static void import_buddies_server_results(struct gaim_connection *gc, gchar *web
 			continue;
 		}
 
-		debug_printf("import_buddies_server_results: uin: %s\n", name);
+		gaim_debug(GAIM_DEBUG_MISC, "gg",
+				   "import_buddies_server_results: uin: %s\n", name);
 		if (!gaim_find_buddy(gc->account, name)) {
 			struct buddy *b;
 			struct group *g;
@@ -822,7 +801,7 @@ static void import_buddies_server_results(struct gaim_connection *gc, gchar *web
 	g_strfreev(users_tbl);
 }
 
-static void export_buddies_server_results(struct gaim_connection *gc, gchar *webdata)
+static void export_buddies_server_results(GaimConnection *gc, gchar *webdata)
 {
 	if (strstr(webdata, "put_success:")) {
 		gaim_notify_info(gc, NULL,
@@ -831,13 +810,14 @@ static void export_buddies_server_results(struct gaim_connection *gc, gchar *web
 		return;
 	}
 
-	debug_printf("export_buddies_server_results: webdata [%s]\n", webdata);
+	gaim_debug(GAIM_DEBUG_MISC, "gg",
+			   "export_buddies_server_results: webdata [%s]\n", webdata);
 	gaim_notify_error(gc, NULL,
 					  _("Couldn't transfer Buddy List to Gadu-Gadu server"),
 					  NULL);
 }
 
-static void delete_buddies_server_results(struct gaim_connection *gc, gchar *webdata)
+static void delete_buddies_server_results(GaimConnection *gc, gchar *webdata)
 {
 	if (strstr(webdata, "put_success:")) {
 		gaim_notify_info(gc, NULL,
@@ -846,13 +826,14 @@ static void delete_buddies_server_results(struct gaim_connection *gc, gchar *web
 		return;
 	}
 
-	debug_printf("delete_buddies_server_results: webdata [%s]\n", webdata);
+	gaim_debug(GAIM_DEBUG_MISC, "gg",
+			   "delete_buddies_server_results: webdata [%s]\n", webdata);
 	gaim_notify_error(gc, NULL,
 					  _("Couldn't delete Buddy List from Gadu-Gadu server"),
 					  NULL);
 }
 
-static void password_change_server_results(struct gaim_connection *gc, gchar *webdata)
+static void password_change_server_results(GaimConnection *gc, gchar *webdata)
 {
 	if (strstr(webdata, "reg_success:")) {
 		gaim_notify_info(gc, NULL,
@@ -860,7 +841,8 @@ static void password_change_server_results(struct gaim_connection *gc, gchar *we
 		return;
 	}
 
-	debug_printf("password_change_server_results: webdata [%s]\n", webdata);
+	gaim_debug(GAIM_DEBUG_MISC, "gg",
+			   "password_change_server_results: webdata [%s]\n", webdata);
 	gaim_notify_error(gc, NULL,
 					  _("Password couldn't be changed"), NULL);
 }
@@ -868,15 +850,16 @@ static void password_change_server_results(struct gaim_connection *gc, gchar *we
 static void http_results(gpointer data, gint source, GaimInputCondition cond)
 {
 	struct agg_http *hdata = data;
-	struct gaim_connection *gc = hdata->gc;
+	GaimConnection *gc = hdata->gc;
 	char *webdata;
 	int len;
 	char read_data;
 
-	debug_printf("http_results: begin\n");
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "http_results: begin\n");
 
-	if (!g_slist_find(connections, gc)) {
-		debug_printf("search_callback: g_slist_find error\n");
+	if (!g_list_find(gaim_connections_get_all(), gc)) {
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "search_callback: g_slist_find error\n");
 		gaim_input_remove(hdata->inpa);
 		g_free(hdata);
 		close(source);
@@ -906,7 +889,8 @@ static void http_results(gpointer data, gint source, GaimInputCondition cond)
 	gaim_input_remove(hdata->inpa);
 	close(source);
 
-	debug_printf("http_results: type %d, webdata [%s]\n", hdata->type, webdata);
+	gaim_debug(GAIM_DEBUG_MISC, "gg",
+			   "http_results: type %d, webdata [%s]\n", hdata->type, webdata);
 
 	switch (hdata->type) {
 	case AGG_HTTP_SEARCH:
@@ -926,7 +910,8 @@ static void http_results(gpointer data, gint source, GaimInputCondition cond)
 		break;
 	case AGG_HTTP_NONE:
 	default:
-		debug_printf("http_results: unsupported type %d\n", hdata->type);
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "http_results: unsupported type %d\n", hdata->type);
 		break;
 	}
 
@@ -937,14 +922,15 @@ static void http_results(gpointer data, gint source, GaimInputCondition cond)
 static void http_req_callback(gpointer data, gint source, GaimInputCondition cond)
 {
 	struct agg_http *hdata = data;
-	struct gaim_connection *gc = hdata->gc;
+	GaimConnection *gc = hdata->gc;
 	gchar *request = hdata->request;
 	gchar *buf;
 
-	debug_printf("http_req_callback: begin\n");
+	gaim_debug(GAIM_DEBUG_INFO, "gg", "http_req_callback: begin\n");
 
-	if (!g_slist_find(connections, gc)) {
-		debug_printf("http_req_callback: g_slist_find error\n");
+	if (!g_list_find(gaim_connections_get_all(), gc)) {
+		gaim_debug(GAIM_DEBUG_ERROR, "gg",
+				   "http_req_callback: g_slist_find error\n");
 		g_free(request);
 		g_free(hdata);
 		close(source);
@@ -957,7 +943,8 @@ static void http_req_callback(gpointer data, gint source, GaimInputCondition con
 		return;
 	}
 
-	debug_printf("http_req_callback: http request [%s]\n", request);
+	gaim_debug(GAIM_DEBUG_MISC, "gg",
+			   "http_req_callback: http request [%s]\n", request);
 
 	buf = g_strdup_printf("POST %s HTTP/1.0\r\n"
 			      "Host: %s\r\n"
@@ -986,11 +973,11 @@ static void http_req_callback(gpointer data, gint source, GaimInputCondition con
 	hdata->inpa = gaim_input_add(source, GAIM_INPUT_READ, http_results, hdata);
 }
 
-static void import_buddies_server(struct gaim_connection *gc)
+static void import_buddies_server(GaimConnection *gc)
 {
 	struct agg_http *hi = g_new0(struct agg_http, 1);
-	gchar *u = gg_urlencode(gc->username);
-	gchar *p = gg_urlencode(gc->password);
+	gchar *u = gg_urlencode(gaim_account_get_username(gc->account));
+	gchar *p = gg_urlencode(gaim_account_get_password(gc->account));
 
 	hi->gc = gc;
 	hi->type = AGG_HTTP_USERLIST_IMPORT;
@@ -1001,7 +988,7 @@ static void import_buddies_server(struct gaim_connection *gc)
 	g_free(u);
 	g_free(p);
 
-	if (proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, hi) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, hi) < 0) {
 		gaim_notify_error(gc, NULL,
 						  _("Unable to import Gadu-Gadu buddy list"),
 						  _("Gaim was unable to connect to the Gadu-Gadu "
@@ -1012,12 +999,12 @@ static void import_buddies_server(struct gaim_connection *gc)
 	}
 }
 
-static void export_buddies_server(struct gaim_connection *gc)
+static void export_buddies_server(GaimConnection *gc)
 {
 	struct agg_http *he = g_new0(struct agg_http, 1);
 	gchar *ptr;
-	gchar *u = gg_urlencode(gc->username);
-	gchar *p = gg_urlencode(gc->password);
+	gchar *u = gg_urlencode(gaim_account_get_username(gc->account));
+	gchar *p = gg_urlencode(gaim_account_get_password(gc->account));
 
 	GaimBlistNode *gnode, *bnode;
 
@@ -1071,7 +1058,7 @@ static void export_buddies_server(struct gaim_connection *gc)
 		}
 	}
 
-	if (proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, he) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, he) < 0) {
 		gaim_notify_error(gc, NULL,
 						  _("Couldn't export buddy list"),
 						  _("Gaim was unable to connect to the buddy "
@@ -1082,11 +1069,11 @@ static void export_buddies_server(struct gaim_connection *gc)
 	}
 }
 
-static void delete_buddies_server(struct gaim_connection *gc)
+static void delete_buddies_server(GaimConnection *gc)
 {
 	struct agg_http *he = g_new0(struct agg_http, 1);
-	gchar *u = gg_urlencode(gc->username);
-	gchar *p = gg_urlencode(gc->password);
+	gchar *u = gg_urlencode(gaim_account_get_username(gc->account));
+	gchar *p = gg_urlencode(gaim_account_get_password(gc->account));
 
 	he->gc = gc;
 	he->type = AGG_HTTP_USERLIST_DELETE;
@@ -1094,7 +1081,7 @@ static void delete_buddies_server(struct gaim_connection *gc)
 	he->host = GG_PUBDIR_HOST;
 	he->request = g_strdup_printf("FmNum=%s&Pass=%s&Delete=1", u, p);
 
-	if (proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, he) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, he) < 0) {
 		gaim_notify_error(gc, NULL,
 						  _("Unable to delete Gadu-Gadu buddy list"),
 						  _("Gaim was unable to connect to the buddy "
@@ -1105,7 +1092,7 @@ static void delete_buddies_server(struct gaim_connection *gc)
 	}
 }
 
-static void agg_dir_search(struct gaim_connection *gc, const char *first, const char *middle,
+static void agg_dir_search(GaimConnection *gc, const char *first, const char *middle,
 			   const char *last, const char *maiden, const char *city, const char *state,
 			   const char *country, const char *email)
 {
@@ -1143,7 +1130,7 @@ static void agg_dir_search(struct gaim_connection *gc, const char *first, const
 		g_free(enew_city);
 	}
 
-	if (proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, srch) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, srch) < 0) {
 		gaim_notify_error(gc, NULL,
 						  _("Unable to access directory"),
 						  _("Gaim was unable to search the Directory "
@@ -1155,11 +1142,11 @@ static void agg_dir_search(struct gaim_connection *gc, const char *first, const
 	}
 }
 
-static void agg_change_passwd(struct gaim_connection *gc, const char *old, const char *new)
+static void agg_change_passwd(GaimConnection *gc, const char *old, const char *new)
 {
 	struct agg_http *hpass = g_new0(struct agg_http, 1);
-	gchar *u = gg_urlencode(gc->username);
-	gchar *p = gg_urlencode(gc->password);
+	gchar *u = gg_urlencode(gaim_account_get_username(gc->account));
+	gchar *p = gg_urlencode(gaim_account_get_password(gc->account));
 	gchar *enew = gg_urlencode(new);
 	gchar *eold = gg_urlencode(old);
 
@@ -1177,7 +1164,7 @@ static void agg_change_passwd(struct gaim_connection *gc, const char *old, const
 	g_free(enew);
 	g_free(eold);
 
-	if (proxy_connect(gc->account, GG_REGISTER_HOST, GG_REGISTER_PORT, http_req_callback, hpass) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_REGISTER_HOST, GG_REGISTER_PORT, http_req_callback, hpass) < 0) {
 	       	gaim_notify_error(gc, NULL,
 							  _("Unable to change Gadu-Gadu password"),
 							  _("Gaim was unable to change your password "
@@ -1190,7 +1177,7 @@ static void agg_change_passwd(struct gaim_connection *gc, const char *old, const
 	}                                        
 }
 
-static GList *agg_actions(struct gaim_connection *gc)
+static GList *agg_actions(GaimConnection *gc)
 {
 	GList *m = NULL;
 	struct proto_actions_menu *pam;
@@ -1232,7 +1219,7 @@ static GList *agg_actions(struct gaim_connection *gc)
 	return m;
 }
 
-static void agg_get_info(struct gaim_connection *gc, const char *who)
+static void agg_get_info(GaimConnection *gc, const char *who)
 {
 	struct agg_http *srch = g_new0(struct agg_http, 1);
 	srch->gc = gc;
@@ -1255,7 +1242,7 @@ static void agg_get_info(struct gaim_connection *gc, const char *who)
 	} else
 		srch->request = g_strdup_printf("Mode=3&UserId=%s", who);
 
-	if (proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, srch) < 0) {
+	if (gaim_proxy_connect(gc->account, GG_PUBDIR_HOST, GG_PUBDIR_PORT, http_req_callback, srch) < 0) {
 		gaim_notify_error(gc, NULL,
 						  _("Unable to access user profile."),
 						  _("Gaim was unable to access this user's "
@@ -1267,7 +1254,7 @@ static void agg_get_info(struct gaim_connection *gc, const char *who)
 	}
 }
 
-static const char *agg_list_icon(struct gaim_account *a, struct buddy *b)
+static const char *agg_list_icon(GaimAccount *a, struct buddy *b)
 {
 	return "gadu-gadu";
 }
@@ -1284,19 +1271,19 @@ static void agg_list_emblems(struct buddy *b, char **se, char **sw, char **nw, c
 		/* Drop all masks */
 		status &= ~(GG_STATUS_FRIENDS_MASK);
 		if (status == GG_STATUS_BUSY)
-			*se = "busy";
+			*se = "away";
 		else if (status == GG_STATUS_INVISIBLE)
 			*se = "invisiible";
 	}
 }
 
 
-static void agg_set_permit_deny_dummy(struct gaim_connection *gc)
+static void agg_set_permit_deny_dummy(GaimConnection *gc)
 {
 	/* It's implemented on client side because GG server doesn't support this */
 }
 
-static void agg_permit_deny_dummy(struct gaim_connection *gc, const char *who)
+static void agg_permit_deny_dummy(GaimConnection *gc, const char *who)
 {
 	/* It's implemented on client side because GG server doesn't support this */
 }
@@ -1374,7 +1361,7 @@ static GaimPluginInfo info =
 	                                                  /**  description    */
 	N_("Gadu-Gadu Protocol Plugin"),
 	"Arkadiusz Mikiewicz <misiek@pld.ORG.PL>",       /**< author         */
-	WEBSITE,                                          /**< homepage       */
+	GAIM_WEBSITE,                                     /**< homepage       */
 
 	NULL,                                             /**< load           */
 	NULL,                                             /**< unload         */
@@ -1385,17 +1372,16 @@ static GaimPluginInfo info =
 };
 
 static void
-__init_plugin(GaimPlugin *plugin)
+init_plugin(GaimPlugin *plugin)
 {
-	struct proto_user_opt *puo;
+	GaimAccountOption *option;
 
-	puo = g_new0(struct proto_user_opt, 1);
-	puo->label = g_strdup("Nick:");
-	puo->def   = g_strdup("Gadu-Gadu User");
-	puo->pos   = USEROPT_NICK;
-	prpl_info.user_opts = g_list_append(prpl_info.user_opts, puo);
+	option = gaim_account_option_string_new(_("Nick"), "nick",
+											"Gadu-Gadu User");
+	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options,
+											   option);
 
 	my_protocol = plugin;
 }
 
-GAIM_INIT_PLUGIN(gg, __init_plugin, info);
+GAIM_INIT_PLUGIN(gg, init_plugin, info);
