@@ -1,4 +1,4 @@
-/* $Id: libgg.c,v 1.10 2002/11/29 12:48:01 lschiere Exp $ */
+/* $Id: libgg.c,v 1.15 2003/07/20 19:11:13 thekingant Exp $ */
 
 /*
  *  (C) Copyright 2001 Wojtek Kaniewski <wojtekka@irc.pl>,
@@ -50,9 +50,10 @@
 #  define WORDS_BIGENDIAN 1
 #endif
 #include "libgg.h"
-#include "config.h"
-#include "gaim.h"
+
+#include "internal.h"
 #include "proxy.h"
+#include "debug.h"
 
 #ifdef _WIN32
 #include "win32dep.h"
@@ -72,7 +73,7 @@ static char rcsid[]
 #ifdef __GNUC__
 __attribute__ ((unused))
 #endif
-= "$Id: libgg.c,v 1.10 2002/11/29 12:48:01 lschiere Exp $";
+= "$Id: libgg.c,v 1.15 2003/07/20 19:11:13 thekingant Exp $";
 
 #endif 
 
@@ -99,6 +100,8 @@ static inline unsigned long fix32(unsigned long x)
  *
  * dla maszyn big-endianowych zamienia kolejno bajtw w ,,short''ach.
  */
+
+/* not currently used
 static inline unsigned short fix16(unsigned short x)
 {
 #ifndef WORDS_BIGENDIAN
@@ -109,6 +112,7 @@ static inline unsigned short fix16(unsigned short x)
                  ((x & (unsigned short) 0xff00U) >> 8));
 #endif
 }
+*/
 
 #ifndef _WIN32
 /*
@@ -249,7 +253,7 @@ static void *gg_recv_packet(struct gg_session *sess)
 				return NULL;
 			}
 			if (errno != EINTR) {
-//				errno = EINVAL;
+				/* errno = EINVAL; */
 				free(buf);
 				return NULL;
 			}
@@ -525,7 +529,7 @@ void gg_logoff(struct gg_session *sess)
  *
  * w przypadku bdu zwraca -1, inaczej numer sekwencyjny.
  */
-int gg_send_message(struct gg_session *sess, int msgclass, uin_t recipient, unsigned char *message)
+int gg_send_message(struct gg_session *sess, int msgclass, uin_t recipient, char *message)
 {
 	struct gg_send_msg s;
 
@@ -578,7 +582,8 @@ int gg_ping(struct gg_session *sess)
 	gg_debug(GG_DEBUG_FUNCTION, "** gg_ping(...);\n");
 	
 	if(ping_outstanding) {
-		debug_printf("Trying to send ping, when we havn't been ponged on last ping\n");
+		gaim_debug(GAIM_DEBUG_INFO, "gg",
+				   "Trying to send ping, when we havn't been ponged on last ping\n");
 		return 1;
 	}
 	else {
@@ -745,7 +750,7 @@ static int gg_watch_fd_connected(struct gg_session *sess, struct gg_event *e)
 		return -1;
 	}
 
-	p = (void*) h + sizeof(struct gg_header);
+	p = (void *)h + sizeof(struct gg_header);
 	
 	if (h->type == GG_RECV_MSG) {
 		struct gg_recv_msg *r = p;
@@ -858,7 +863,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 	switch (sess->state) {
 #ifndef _WIN32
 		/* Apparantly we will never be in this state as long as we are
-		   using proxy_connect instead of gg_login - Herman */
+		   using gaim_proxy_connect instead of gg_login - Herman */
 		case GG_STATE_RESOLVING:
 		{
 			struct in_addr a;
@@ -921,7 +926,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 		case GG_STATE_CONNECTING:
 		{
 			char buf[1024];
-			int res, res_size = sizeof(res);
+			unsigned int res, res_size = sizeof(res);
 
 			gg_debug(GG_DEBUG_MISC, "== GG_STATE_CONNECTING\n");
 
@@ -1045,7 +1050,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			a.s_addr = inet_addr(host);
 			sess->server_ip = a.s_addr;
 #if 0
-			/* We need to watch this non-blocking socket so lets use proxy_connect 
+			/* We need to watch this non-blocking socket so lets use gaim_proxy_connect 
 			   in gg.c - Herman */
 			if((sess->fd = gg_connect(&a, port, sess->assync)) == -1) {
 				gg_debug(GG_DEBUG_MISC, "-- connection failed, trying https connection\n");
@@ -1069,7 +1074,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 
 		case GG_STATE_CONNECTING_GG:
 		{
-			int res, res_size = sizeof(res);
+			unsigned int res, res_size = sizeof(res);
 
 			gg_debug(GG_DEBUG_MISC, "== GG_STATE_CONNECTING_GG\n");
 
@@ -1126,7 +1131,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 				break;
 			}
 	
-			w = (void*) h + sizeof(struct gg_header);
+			w = (struct gg_welcome *)((void *)h + sizeof(struct gg_header));
 			w->key = fix32(w->key);
 
 			for (hash = 1; *password; password++)
