@@ -214,29 +214,9 @@ static int aim_ssi_itemlist_cmp(struct aim_ssi_item *cur1, struct aim_ssi_item *
 	if (!cur1->data && cur2->data)
 		return 3;
 
-	if (cur1->data && cur2->data) {
-		/* Write each TLV list to a bstream and then memcmp them */
-		aim_bstream_t bs1, bs2;
-
-		if (aim_sizetlvchain(&cur1->data) != aim_sizetlvchain(&cur2->data))
+	if ((cur1->data && cur2->data) && (aim_tlvlist_cmp(cur1->data, cur2->data)))
 			return 4;
 
-		aim_bstream_init(&bs1, ((fu8_t *)malloc(aim_sizetlvchain(&cur1->data)*sizeof(fu8_t))), aim_sizetlvchain(&cur1->data));
-		aim_bstream_init(&bs2, ((fu8_t *)malloc(aim_sizetlvchain(&cur2->data)*sizeof(fu8_t))), aim_sizetlvchain(&cur2->data));
-
-		aim_writetlvchain(&bs1, &cur1->data);
-		aim_writetlvchain(&bs2, &cur2->data);
-
-		if (memcmp(bs1.data, bs2.data, bs1.len)) {
-			free(bs1.data);
-			free(bs2.data);
-			return 4;
-		}
-
-		free(bs1.data);
-		free(bs2.data);
-	}
-
 	if (cur1->name && !cur2->name)
 		return 5;
 
@@ -1023,7 +1003,7 @@ faim_export int aim_ssi_setpermdeny(aim_session_t *sess, fu8_t permdeny, fu32_t
  *
  * @param sess The oscar session.
  * @param iconcsum The MD5 checksum of the icon you are using.
- * @param iconcsumlen Length of the MD5 checksum given above.  Should be 10 bytes.
+ * @param iconcsumlen Length of the MD5 checksum given above.  Should be 0x10 bytes.
  * @return Return 0 if no errors, otherwise return the error number.
  */
 faim_export int aim_ssi_seticon(aim_session_t *sess, fu8_t *iconsum, fu16_t iconsumlen)
@@ -1035,30 +1015,36 @@ faim_export int aim_ssi_seticon(aim_session_t *sess, fu8_t *iconsum, fu16_t icon
 	if (!sess || !iconsum || !iconsumlen)
 		return -EINVAL;
 
-	/* Create the data for the TLV containing the icon checksum */
 	if (!(csumdata = (fu8_t *)malloc((iconsumlen+2)*sizeof(fu8_t))))
 		return -ENOMEM;
 	csumdata[0] = 0x00;
 	csumdata[1] = 0x10;
 	memcpy(&csumdata[2], iconsum, iconsumlen);
 
-	/* Need to add the x0131 TLV to the TLV chain */
+	/* Need to add the x00d5 TLV to the TLV chain */
+	aim_addtlvtochain_raw(&data, 0x00d5, (iconsumlen+2) * sizeof(fu8_t), csumdata);
+
+	/* This TLV is added to cache the icon. */
 	aim_addtlvtochain_noval(&data, 0x0131);
 
-	/* Need to add the x00d5 TLV to the TLV chain */
-	aim_addtlvtochain_raw(&data, 0x00d5, 0x0012, csumdata);
-
-	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, "0", AIM_SSI_TYPE_ICONINFO))) {
+	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, "1", AIM_SSI_TYPE_ICONINFO))) {
+		/* If the new tlvchain and oldtlvchain are the same, then do nothing */
+		if (!aim_tlvlist_cmp(tmp->data, data)) {
+			/* The new tlvlist is the identical to the old one */
+			aim_freetlvchain(&data);
+			free(csumdata);
+			return 0;
+		}
 		aim_freetlvchain(&tmp->data);
 		tmp->data = data;
 	} else {
-		tmp = aim_ssi_itemlist_add(&sess->ssi.local, "0", 0x0000, 0xFFFF, AIM_SSI_TYPE_ICONINFO, data);
+		tmp = aim_ssi_itemlist_add(&sess->ssi.local, "1", 0x0000, 0x51F4, AIM_SSI_TYPE_ICONINFO, data);
 		aim_freetlvchain(&data);
 	}
 
 	/* Sync our local list with the server list */
 	aim_ssi_sync(sess);
-
+	free(csumdata);
 	return 0;
 }
 
@@ -1148,17 +1134,26 @@ static int parserights(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx,
 }
 
 /*
- * Subtype 0x0004 - Request SSI Data.
- * XXX - If you don't have a timestamp and revision number?
- *
- * Note that the client should never increment the revision, only the server.
+ * Subtype 0x0004 - Request SSI Data when you don't have a timestamp and 
+ * revision number.
  * 
  */
+faim_export int aim_ssi_reqdata(aim_session_t *sess)
+{
+	aim_conn_t *conn;
 
+	if (!sess || !(conn = aim_conn_findbygroup(sess, AIM_CB_FAM_SSI)))
+		return -EINVAL;
+
+	/* Free any current data, just in case */
+	aim_ssi_freelist(sess);
+
+	return aim_genericreq_n_snacid(sess, conn, AIM_CB_FAM_SSI, AIM_CB_SSI_REQDATA);
+}
 
 /*
- * Subtype 0x0005 - Request SSI Data.
- * XXX - If you have a timestamp and revision number?
+ * Subtype 0x0005 - Request SSI Data when you have a timestamp and revision 
+ * number.
  *
  * The data will only be sent if it is newer than the posted local
  * timestamp and revision.
@@ -1166,7 +1161,7 @@ static int parserights(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx,
  * Note that the client should never increment the revision, only the server.
  * 
  */
-faim_export int aim_ssi_reqdata(aim_session_t *sess, time_t timestamp, fu16_t numitems)
+faim_export int aim_ssi_reqifchanged(aim_session_t *sess, time_t timestamp, fu16_t numitems)
 {
 	aim_conn_t *conn;
 	aim_frame_t *fr;
@@ -1178,9 +1173,9 @@ faim_export int aim_ssi_reqdata(aim_session_t *sess, time_t timestamp, fu16_t nu
 	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_FLAP, 0x02, 10+4+2)))
 		return -ENOMEM;
 
-	snacid = aim_cachesnac(sess, AIM_CB_FAM_SSI, AIM_CB_SSI_REQLIST, 0x0000, NULL, 0);
+	snacid = aim_cachesnac(sess, AIM_CB_FAM_SSI, AIM_CB_SSI_REQIFCHANGED, 0x0000, NULL, 0);
 
-	aim_putsnac(&fr->data, AIM_CB_FAM_SSI, AIM_CB_SSI_REQLIST, 0x0000, snacid);
+	aim_putsnac(&fr->data, AIM_CB_FAM_SSI, AIM_CB_SSI_REQIFCHANGED, 0x0000, snacid);
 	aimbs_put32(&fr->data, timestamp);
 	aimbs_put16(&fr->data, numitems);
 
@@ -1572,7 +1567,7 @@ static int parseack(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 /*
  * Subtype 0x000f - SSI Data Unchanged.
  *
- * Response to aim_ssi_reqdata() if the server-side data is not newer than
+ * Response to aim_ssi_reqifchanged() if the server-side data is not newer than
  * posted local stamp/revision.
  *
  */
